@@ -1,6 +1,7 @@
-/* animated.h
-   animated objects
-
+/** \file animated.h
+	The base class for all Animated (ie. drawable) objects.
+*/
+/*
    Copyright (C) 2000  Mathias Broxvall
 
    This program is free software; you can redistribute it and/or modify
@@ -30,15 +31,19 @@ class Animated : public GameHook {
   Animated();
 
   virtual ~Animated();
-  virtual void draw();
-  virtual void draw2(); /* For eg. transparent things */
+  virtual void draw();                 /**< First drawing pass of object. Only opaque objects. */
+  virtual void draw2();                /**< Draws the second pass (eg. alpha) of object. */
+  virtual void computeBoundingBox();   /**< Recomputes the bounding box of the object. Needed after changes in size */
   void onRemove();
   int flags;            /* General purpose field. Semantics defined by children */
 
   // Tries to make the current position reasonable (eg. set to height of map etc.)
   virtual void has_moved(); 
 
-  Coord3d position;
+  int onScreen;                        /**< Computed by game::draw if the object appears roughly on the screen or not. */
+  Coord3d position;                    /**< Center position of object */
+  Coord3d boundingBox[2];              /**< Lower/higher coordinate of boundingbox relative to position */
+
   GLfloat primaryColor[4],secondaryColor[4],specularColor[4];
   GLuint texture;
 };
