@@ -33,13 +33,12 @@
 #define MOD_DIZZY      4
 #define MOD_FROZEN     5
 #define MOD_FLOAT      6
-// this is not real mod. Used only for special modPill's
-#define MOD_EXTRA_LIFE 7
-
+#define MOD_EXTRA_LIFE 7    // this is not a real mod.
 #define MOD_SMALL      8
 #define MOD_LARGE      9
+#define MOD_NITRO      10
 
-#define NUM_MODS       10
+#define NUM_MODS       11
 
 #define DIE_CRASH  0
 #define DIE_FF     1
@@ -55,7 +54,9 @@ class Ball : public Animated {
   void draw();
   void draw2();
   void tick(Real);
+  void doExpensiveComputations();
   void onRemove();
+  void setReflectivity(double reflectivity,int metallic);
   virtual Boolean crash(Real speed);             // Generic crash called when we hit ground etc. Returns if we are still alive
   virtual Boolean crash(Real speed,Ball *);      // When we crash into another ball. Returns if we are still alive
   virtual void die(int how)=0;
@@ -73,15 +74,22 @@ class Ball : public Animated {
   Real friction,gravity,bounceFactor;
   Real crashTolerance;
   int resolution,no_physics;
+  int ballResolution;              /* 0 for low resolution balls (eg. debris), 1 for normal balls, 2 for high res balls (eg. player) */
 
   static const Real physicsResolution;
   static class std::set<Ball *> *balls;  
 
   double sink; // how far the ball has sunk into acid / sand / etc.
   Matrix4d rotations;
+
+  GLuint environmentTexture;
  protected:
+  double nitroDebrisCount;
  private:
   Boolean checkGroundCollisions(class Map *,Real x,Real y);
+  double reflectivity;
+  int metallic,dontReflectSelf;
+
 
   static GLuint dizzyTexture;
   static GLfloat dizzyTexMinX, dizzyTexMinY;
