@@ -27,7 +27,9 @@ class Game {
   virtual ~Game();
 
   void tick(Real);
-  void draw();
+  void draw();                                   /* Used for normal drawing mode */
+  void drawReflection(Coord3d focus);            /* Used when drawing environmentmaps */
+  void doExpensiveComputations();
   void add(class Animated *);
   void add(class GameHook *);
   void remove(class Animated *);
@@ -52,6 +54,7 @@ class Game {
   char levelName[256];
   char nextLevel[256]; 
   char returnLevel[256]; /* When returning from bonus levels */
+
  protected:
  private:
   void setDefaults();
