@@ -30,8 +30,17 @@
 using namespace std;
 
 Gamer::Gamer() {
-  sprintf(name,"John Doe");
-  if(NULL != getenv("USER")) snprintf(name,20,getenv("USER"));  
+  strcpy(name,"John Doe");
+  if(NULL != getenv("USER")){
+    snprintf(name,20,"%s",getenv("USER"));
+  }
+#ifdef WIN32
+  else{
+    if(NULL != getenv("USERNAME")){
+      snprintf(name,20,"%s",getenv("USERNAME"));
+    }
+  }
+#endif
 
   for(int i=0;i<Settings::settings->nLevelSets;i++) 
 	levels[i] = new KnownLevel[256];
@@ -81,9 +90,17 @@ void Gamer::save() {
   Settings *settings=Settings::settings;
 
   snprintf(str,sizeof(str)-1,"%s/.trackballs",getenv("HOME"));
-
-  mkdir(str,S_IXUSR|S_IRUSR|S_IWUSR|S_IXGRP|S_IRGRP|S_IWGRP);
+  if(pathIsLink(str)) { 
+	fprintf(stderr,_("Error, %s/.trackballs is a symbolic link. Cannot save settings\n"),getenv("HOME"));
+	return;
+  }
+  if(!pathIsDir(str)) 
+	mkdir(str,S_IXUSR|S_IRUSR|S_IWUSR|S_IXGRP|S_IRGRP|S_IWGRP);
   snprintf(str,sizeof(str)-1,"%s/.trackballs/%s.gmr",getenv("HOME"),name);
+  if(pathIsLink(str)) { 
+	fprintf(stderr,_("Error, %s/.trackballs/%s.gmr is a symbolic link. Cannot save settings\n"),getenv("HOME"),name);
+	return;
+  }
 
   gzFile gp = gzopen(str,"wb9");
   if(gp) {
