@@ -1,8 +1,9 @@
-/* glHelp.cc
+/** \file glHelp.cc
    Some OpenGL utility algorithms.
-
-   Copyright (C) 2000  Mathias Broxvall
-   		       Yannick Perret
+*/
+/*
+   Copyright (C) 2000-2005  Mathias Broxvall
+                            Yannick Perret
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,11 +23,11 @@
 #include "general.h"
 #include "SDL/SDL_ttf.h"
 #include "SDL/SDL_opengl.h"
-#include <sys/time.h>
 #include "settings.h"
 #include "font.h"
 #include "SDL/SDL_image.h"
 #include "sparkle2d.h"
+#include "image.h"
 
 using namespace std;
 
@@ -36,7 +37,9 @@ int screenWidth=640, screenHeight=480;
 const GLfloat white[4] = {1.0,1.0,1.0,1.0};
 const GLfloat black[4] = {0.0,0.0,0.0,1.0};
 GLUquadricObj *qobj,*qball,*qdiamond;
-GLuint loresSphere,hiresSphere;
+
+/* Precompiled display lists for spheres of varying resolution (dependent on gfx settings) */
+GLuint sphereDisplayLists[3];
 
 TTF_Font *msgFont, *infoFont, *ingameFont, *menuFont, *scrollFont;
 extern SDL_Surface *screen;
@@ -51,7 +54,10 @@ char *textureNames[GLHELP_MAX_TEXTURES] = {NULL};
 int numTextures;
 
 Sparkle2D *sparkle2D=NULL;
+/*
 SDL_Surface *mousePointer;
+GLuint mousePointerTexture;
+*/
 
 void draw2DString(TTF_Font *font,char *string, int x,int y,Uint8 red, Uint8 green, Uint8 blue) {
   int w,h;
@@ -90,13 +96,19 @@ void draw2DString(TTF_Font *font,char *string, int x,int y,Uint8 red, Uint8 gree
   glDeleteTextures(1,&texture);
   glBindTexture(GL_TEXTURE_2D, textures[0]); // This is needed for mga_dri cards
 }
+/* Creates a texture from an SDL surface and draws it on screen. 
+   Inefficient since the texture is created + deleted.  Avoid using this function.
+*/
 void drawSurface(SDL_Surface *surface,int x,int y,int w,int h) {
-  GLuint texture;
+  static GLuint texture=0;
   GLfloat texcoord[4];
   GLfloat texMinX, texMinY;
-  GLfloat texMaxX, texMaxY;
+  GLfloat texMaxX, texMaxY;  
+
+  
+  if(texture == 0) texture = LoadTexture(surface,texcoord,0,&texture);
+  else texture = LoadTexture(surface,texcoord,0,&texture);
 
-  texture = LoadTexture(surface,texcoord);
   texMinX = texcoord[0];
   texMinY = texcoord[1];
   texMaxX = texcoord[2];
@@ -111,11 +123,12 @@ void drawSurface(SDL_Surface *surface,int x,int y,int w,int h) {
   glTexCoord2f(texMaxX, texMaxY); glVertex2i(x+w, y+h);
   glEnd();
   Leave2DMode();
-  glDeleteTextures(1,&texture);
   glBindTexture(GL_TEXTURE_2D, textures[0]); // This is needed for mga_dri cards
+  //glDeleteTextures(1,&texture);
 }
 inline Real abs(Real v) {return v>0.0?v:-v;}
 double mousePointerPhase=0.0;
+
 void tickMouse(Real td) {
   int mouseX,mouseY;
   static int oldMouseX=0,oldMouseY=0;
@@ -137,7 +150,7 @@ void tickMouse(Real td) {
 
   mousePointerPhase += td;
   sparkle2D->tick(td);
-  last_sparkle += td * (1.0+(abs(mouseSpeedX)+abs(mouseSpeedY))*0.1);
+  last_sparkle += td * (1.0+(fabs(mouseSpeedX)+fabs(mouseSpeedY))*0.1);
   while(last_sparkle > 0.0) {
 	last_sparkle -= 0.05;
 	sparkle2D->add((float)mouseX+10.0*(frandom()-0.5),
@@ -149,31 +162,43 @@ void tickMouse(Real td) {
 				   0.8+0.2*frandom(),0.8+0.2*frandom(),0.5+0.2*frandom(),0.9-0.35*frandom());
   }
 }
+/** Draws a textured 2D rectangle with min texcoord at x, y, max texcoord at x+w,y+h.
+   Uses the texture which is already bound.
+   Suitable for drawing 2D graphics like the panels on the screen.
+*/
+void drawTextured2DRectangle(int x,int y,int w,int h) {
+  glBegin(GL_TRIANGLE_STRIP);
+  glTexCoord2f(0.0, 0.0); glVertex2i(x,   y  );
+  glTexCoord2f(1.0, 0.0); glVertex2i(x+w, y  );
+  glTexCoord2f(0.0, 1.0); glVertex2i(x,   y+h);
+  glTexCoord2f(1.0, 1.0); glVertex2i(x+w, y+h);
+  glEnd();
+}
+
 void drawMousePointer() {
   int mouseX,mouseY;
   glColor4f(1.0,1.0,1.0,1.0);
   sparkle2D->draw();
   Uint8 mouseState=SDL_GetMouseState(&mouseX,&mouseY);
-  drawMouse(mousePointer,mouseX-32,mouseY-32,64,64, 0.01);
+  drawMouse(mouseX,mouseY,64,64, 0.01);  
 }
-void drawMouse(SDL_Surface *surface,int x,int y,int w,int h, Real td) {
-  GLuint texture;
-  GLfloat texcoord[4];
+
+void drawMouse(int x,int y,int w,int h, Real td) {
   GLfloat texMinX, texMinY;
   GLfloat texMaxX, texMaxY;
 
-  texture = LoadTexture(surface,texcoord);
-  texMinX = texcoord[0];
-  texMinY = texcoord[1];
-  texMaxX = texcoord[2];
-  texMaxY = texcoord[3];  
+  bindTexture("mousePointer.png");
+  texMinX = 0.0;
+  texMinY = 0.0;
+  texMaxX = 1.0;
+  texMaxY = 1.0;
 
   Enter2DMode();
   glEnable(GL_BLEND);
   glColor4f(1.0,1.0, 1.0, 1.0); //0.8 + 0.2*cos(3.23*mousePointerPhase));
-  glBindTexture(GL_TEXTURE_2D, texture);
+  //glBindTexture(GL_TEXTURE_2D, mousePointerTexture);
   glPushMatrix();
-  glTranslatef(x+w/2.0,y+h/2.0,0.0);
+  glTranslatef(x,y,0.0);
   glRotatef(mousePointerPhase*20.0, 0.0,0.0,1.0);
   glScalef(1.0+0.1*cos(mousePointerPhase*1.8),1.0+0.1*cos(mousePointerPhase*1.9),1.0);
   glBegin(GL_TRIANGLE_STRIP);
@@ -185,7 +210,7 @@ void drawMouse(SDL_Surface *surface,int x,int y,int w,int h, Real td) {
   glPopMatrix();
   glDisable(GL_BLEND);
   Leave2DMode();
-  glDeleteTextures(1,&texture);
+  //glDeleteTextures(1,&texture);
   glBindTexture(GL_TEXTURE_2D, textures[0]); // This is needed for mga_dri cards
 }
 void drawFuzzySurface(SDL_Surface *surface,int x,int y,int w,int h) {
@@ -254,7 +279,7 @@ int createSnapshot()
   /* allocate buffer */
   if ((buffer = (unsigned char*)malloc(sizeof(unsigned char)*screenWidth*screenHeight*3)) == NULL)
     {
-    fprintf(stderr, "Warning: cannot allocate %d bytes for snapshot. Abort.\n", sizeof(unsigned char)*screenWidth*screenHeight*3);
+    fprintf(stderr, _("Warning: cannot allocate %d bytes for snapshot. Aborting.\n"), sizeof(unsigned char)*screenWidth*screenHeight*3);
     return(0);
     }
 
@@ -273,17 +298,23 @@ int createSnapshot()
     }
   while(again);
 
+  /* Check against symlinks */
+  if(pathIsLink(name)) {
+	fprintf(stderr, _("Error: file %s is a symlink.\n"), name);
+	return 0;
+  }
+
+
   /* get the screen */
   glReadPixels(0, 0, screenWidth, screenHeight, GL_RGB,
                GL_UNSIGNED_BYTE, (GLvoid*)buffer);
 
   /* save the image */
-  if ((f = fopen(name, "w")) == NULL)
-    {
-    fprintf(stderr, "Warning: cannot create file '%s'. Abort\n", name);
+  if ((f = fopen(name, "w")) == NULL) {
+    fprintf(stderr, _("Warning: cannot create file '%s'. Abort\n"), name);
     free(buffer);
     return(0);
-    }
+  }
 
   fprintf(f, "P6\n# CREATOR: Hexasoft Corp. / trackballs Snapshot\n");
   fprintf(f, "%d %d\n255\n", screenWidth, screenHeight);
@@ -400,66 +431,106 @@ void drawTriangle(Coord3d a,Coord3d b,Coord3d c) {
   glEnd();
 }
 
-int loadTexture(char *name) {
+SDL_Surface *getSurfaceFromRGB(char *fullPath){
+  SDL_Surface *surface;
+  int tx, ty;
+  unsigned char *font_data=NULL;
+
+  read_2d_image_rgb(fullPath, &font_data, &tx, &ty);
+  surface = SDL_CreateRGBSurfaceFrom((void *)font_data, 
+				     tx, 
+				     ty, 
+				     32, 
+				     tx*4,
+				     0xFF000000,
+				     0x00FF0000,
+				     0x0000FF00,
+				     0x000000FF);
+
+  if( ! surface )
+    printf(_("Error creating surface from RGB file: %s\n"),
+	   SDL_GetError());
+  return surface;
+}
+
+/** Loads a texture from file and returns a reference to it.
+	It is safe to load the same texture multiple times since the results are cached 
+*/
+int loadTexture(char *name, Font *font) {
   GLfloat texCoord[4];
   char str[256];
   SDL_Surface *surface;
-  
+  char *tempStr;  
   int i;
+
+  /* Check in cache if texture already loaded */
   for(i=0;i<numTextures;i++)
 	if(strcmp(textureNames[i],name)==0) return i;
 
   if (numTextures >= GLHELP_MAX_TEXTURES)  {
-  	fprintf(stderr, "Warning: max. number of textures reached (%d). Texture '%s' not loaded.\n",
+  	fprintf(stderr, _("Warning: max. number of textures reached (%d). Texture '%s' not loaded.\n"),
 	    GLHELP_MAX_TEXTURES, name);
 	return 0;
   }
   
-  sprintf(str,"%s/images/%s",SHARE_DIR,name);
-  surface = IMG_Load(str);
+  snprintf(str,sizeof(str),"%s/images/%s",SHARE_DIR,name);
+  tempStr = strstr(name, ".rgb");
+  if( tempStr != NULL && strcmp(tempStr, ".rgb") == 0){
+    surface = getSurfaceFromRGB(str);
+  }else{
+    surface = IMG_Load(str);
+  }
   if(!surface) {
-	printf("Warning: Failed to load %s\n",str);
+	printf(_("Warning: Failed to load texture %s\n"),str);
 	textureNames[numTextures] = strdup(name);
 	textures[numTextures++]=textures[0]; // just assume we managed to load this, better than nothing
-	return 0;  // error (a valid texture entry)
+	return 0;  // BUG (a valid texture entry)
   } else {
 	textureNames[numTextures] = strdup(name);
-	textures[numTextures++] = LoadTexture(surface,texCoord);
-        SDL_FreeSurface(surface);
+	textures[numTextures++] = LoadTexture(surface,texCoord, font != NULL);
+
+	/* Strange special case handling when we are called from the FONT routines */
+	if( font != NULL ){
+	  if( !(font->figureFontInfo(surface->pixels, 
+				    surface->w, 
+				     surface->h)))
+	    {
+	      --numTextures;
+	      free(textureNames[numTextures]);
+	      textureNames[numTextures] = NULL;
+	      glDeleteTextures(1, &textures[numTextures] );
+	      SDL_FreeSurface(surface);
+	      return -1;
+	      
+	    } 
+	}
+    SDL_FreeSurface(surface);
   }
   return (numTextures-1);  // ok
 }
 
 void glHelpInit() {  
   char str[256];
+  GLfloat texCoords[4];
+
   qobj = gluNewQuadric();
   gluQuadricDrawStyle(qobj,GLU_FILL);
 
-  loresSphere = glGenLists(1);
-  glNewList(loresSphere,GL_COMPILE);
-  gluSphere(qobj, 1.0, 6, 6);
-  glEndList();
-
-  hiresSphere = glGenLists(1);
-  glNewList(hiresSphere,GL_COMPILE);
-  gluSphere(qobj, 1.0, 13, 13);
-  glEndList();
-
   for(int i=0;i<4711;i++)
 	fake_rand[i] = frand();
 
   TTF_Init();
-  sprintf(str,"%s/fonts/%s",SHARE_DIR,"menuFont.ttf"); msgFont = TTF_OpenFont(str,30); // 30 astron
-  if(!msgFont) { fprintf(stderr,"Error: Failed to load font %s\n",str); exit(0); }
-  sprintf(str,"%s/fonts/%s",SHARE_DIR,"menuFont.ttf"); infoFont = TTF_OpenFont(str,18);
-  if(!infoFont) { fprintf(stderr,"Error: Failed to load font %s\n",str); exit(0); }
-  sprintf(str,"%s/fonts/%s",SHARE_DIR,"menuFont.ttf"); ingameFont = TTF_OpenFont(str,30); // barbatri
-  if(!ingameFont) { fprintf(stderr,"Error: Failed to load font %s\n",str); exit(0); }
-  sprintf(str,"%s/fonts/%s",SHARE_DIR,"menuFont.ttf"); menuFont = TTF_OpenFont(str,40); // barbatri
-  if(!menuFont) { fprintf(stderr,"Error: Failed to load font %s\n",str); exit(0); }
-  sprintf(str,"%s/fonts/%s",SHARE_DIR,"menuFont.ttf"); scrollFont = TTF_OpenFont(str,18);
+  snprintf(str,sizeof(str),"%s/fonts/%s",SHARE_DIR,"menuFont.ttf"); msgFont = TTF_OpenFont(str,30); // 30 astron
+  if(!msgFont) { fprintf(stderr,_("Error: Failed to load font %s\n"),str); exit(0); }
+  snprintf(str,sizeof(str),"%s/fonts/%s",SHARE_DIR,"menuFont.ttf"); infoFont = TTF_OpenFont(str,18);
+  if(!infoFont) { fprintf(stderr,_("Error: Failed to load font %s\n"),str); exit(0); }
+  snprintf(str,sizeof(str),"%s/fonts/%s",SHARE_DIR,"menuFont.ttf"); ingameFont = TTF_OpenFont(str,30); // barbatri
+  if(!ingameFont) { fprintf(stderr,_("Error: Failed to load font %s\n"),str); exit(0); }
+  snprintf(str,sizeof(str),"%s/fonts/%s",SHARE_DIR,"menuFont.ttf"); menuFont = TTF_OpenFont(str,40); // barbatri
+  if(!menuFont) { fprintf(stderr,_("Error: Failed to load font %s\n"),str); exit(0); }
+  snprintf(str,sizeof(str),"%s/fonts/%s",SHARE_DIR,"menuFont.ttf"); scrollFont = TTF_OpenFont(str,18);
   if(!(msgFont&&infoFont&&ingameFont)) {
-	printf("Error: Failed to load fonts\n");
+	printf(_("Error: Failed to load fonts\n"));
 	exit(0);
   }
   TTF_SetFontStyle(msgFont,TTF_STYLE_NORMAL);
@@ -474,11 +545,16 @@ void glHelpInit() {
   loadTexture("texture3.png");
   loadTexture("texture4.png");
   loadTexture("wings.png");
+  loadTexture("mousePointer.png");
 
   sparkle2D = new Sparkle2D();
+  /*
   sprintf(str,"%s/images/mousePointer.png",SHARE_DIR);
   mousePointer = IMG_Load(str);
   if(!mousePointer) { printf("Error: Failed to load %s\n",str); exit(0); }
+  mousePointerTexture = LoadTexture(mousePointer,);
+  if(!mousePointerTexture) { printf("Error: Failed to allocate texture for mousepointer\n"); exit(0); }
+  */
 
   qball = gluNewQuadric();
   gluQuadricDrawStyle(qball,GLU_FILL);
@@ -489,6 +565,51 @@ void glHelpInit() {
   gluQuadricDrawStyle(qdiamond,GLU_FILL);
   gluQuadricNormals(qdiamond,GLU_SMOOTH);
   gluQuadricTexture(qdiamond,GL_TRUE);
+
+  sphereDisplayLists[0] = glGenLists(1);
+  sphereDisplayLists[1] = glGenLists(1);
+  sphereDisplayLists[2] = glGenLists(1);
+  regenerateSphereDisplaylists();
+}
+
+void regenerateSphereDisplaylists() {
+  int resolution;
+  
+  glNewList(sphereDisplayLists[0],GL_COMPILE);  
+  switch(Settings::settings->gfx_details) {
+  case GFX_DETAILS_NONE: resolution=6; break;
+  case GFX_DETAILS_MINIMALISTIC: resolution=8; break;
+  case GFX_DETAILS_SIMPLE: resolution=8; break;
+  case GFX_DETAILS_NORMAL: resolution=8; break;
+  case GFX_DETAILS_EXTRA: resolution=10; break;
+  case GFX_DETAILS_EVERYTHING: resolution=12; break;
+  }
+  gluSphere(qball, 1.0, resolution, resolution/2);
+  glEndList();
+
+  glNewList(sphereDisplayLists[1],GL_COMPILE);  
+  switch(Settings::settings->gfx_details) {
+  case GFX_DETAILS_NONE: resolution=6; break;
+  case GFX_DETAILS_MINIMALISTIC: resolution=8; break;
+  case GFX_DETAILS_SIMPLE: resolution=10; break;
+  case GFX_DETAILS_NORMAL: resolution=12; break;
+  case GFX_DETAILS_EXTRA: resolution=16; break;
+  case GFX_DETAILS_EVERYTHING: resolution=20; break;
+  }
+  gluSphere(qball, 1.0, resolution, resolution/2);
+  glEndList();
+  
+  glNewList(sphereDisplayLists[2],GL_COMPILE);  
+  switch(Settings::settings->gfx_details) {
+  case GFX_DETAILS_NONE: resolution=8; break;
+  case GFX_DETAILS_MINIMALISTIC: resolution=10; break;
+  case GFX_DETAILS_SIMPLE: resolution=12; break;
+  case GFX_DETAILS_NORMAL: resolution=16; break;
+  case GFX_DETAILS_EXTRA: resolution=20; break;
+  case GFX_DETAILS_EVERYTHING: resolution=24; break;
+  }
+  gluSphere(qball, 1.0, resolution, resolution/2);
+  glEndList();
 }
 
 #define FRAME 50
@@ -506,6 +627,37 @@ int bindTexture(char *name) {
   glBindTexture(GL_TEXTURE_2D,textures[i]);
 }
 
+int resetTextures() {
+  GLfloat texCoord[4];
+  char str[256];
+  SDL_Surface *surface;
+  int i;
+  char *tempStr;
+  int linear = 0;
+
+  for(i=0;i<numTextures;i++){
+
+    if(textureNames[i] == NULL) continue;
+    snprintf(str,sizeof(str),"%s/images/%s",SHARE_DIR,textureNames[i]);
+    tempStr = strstr(textureNames[i], ".rgb");
+    if( tempStr != NULL && strcmp(tempStr, ".rgb") == 0){
+      surface = getSurfaceFromRGB(str);
+      linear = 1;
+    }else{
+      surface = IMG_Load(str);
+    }
+    if(!surface) {
+      printf(_("Warning: Failed to load texture %s\n"),str);
+      return 0;  // error (a valid texture entry)
+    } else {
+      glDeleteTextures(1, &textures[i] );
+      textures[i] = LoadTexture(surface,texCoord, linear);
+      SDL_FreeSurface(surface);
+    }
+  }  
+  
+}
+
 /* Calculates and displays current framerate */
 void displayFrameRate() {
   static double oldTime=0.0;
@@ -518,15 +670,15 @@ void displayFrameRate() {
 	fps = 1.0;
   else if(td <= 1e-4) {
 	static int hasWarned=0;
-	if(!hasWarned) { printf("Warning: td too small %f\n",td); hasWarned=1; }
+	if(!hasWarned) { printf(_("Warning: too fast framerate (%f)\n"),td); hasWarned=1; }
   } else
 	fps = fps * 0.95+ 0.05 / td;
   oldTime = t;
   realTimeNow = t;
   
   if(Settings::settings->showFPS) {
-	if(fps>0) sprintf(str,"Framerate: %.1f",fps);
-	else sprintf(str,"Framerate unknown");
+	if(fps>0) snprintf(str,sizeof(str),_("Framerate: %.1f"),fps);
+	else snprintf(str,sizeof(str),_("Framerate unknown"));
 
 	//glColor3f(1.0,1.0,1.0);
 	Font::drawSimpleText(0, str, 15, screenHeight-15, 10., 10., 1., 1., 0.25, 0.8);
@@ -537,6 +689,7 @@ void displayFrameRate() {
 /* Vector operations */
 /********************/
 
+#ifndef INLINE_VECTOR_OPS
 /* C <- A + B */
 void add(const double A[3],const double B[3],double C[3]) {
   for(int i=0;i<3;i++) C[i] = A[i] + B[i];
@@ -566,6 +719,7 @@ void crossProduct(const double A[3],const double B[3],double C[3]) {
 double dotProduct(const double A[3],const double B[3]) {  
   return A[0]*B[0] + A[1]*B[1] + A[2]*B[2];
 }
+#endif
 
 /*********************/
 /* Matrix operations */
@@ -701,7 +855,7 @@ void Leave2DMode() {
 }
 
 
-int power_of_two(int input) {
+int powerOfTwo(int input) {
 	int value = 1;
 
 	while ( value < input ) {
@@ -710,110 +864,142 @@ int power_of_two(int input) {
 	return value;
 }
 
+GLuint LoadTexture(SDL_Surface *surface, GLfloat *texcoord, int linearFilter,GLuint *texture) {
+  int w, h;
+  SDL_Surface *image;
+  SDL_Rect area;
+  Uint32 saved_flags;
+  Uint8  saved_alpha;
+  int useMipmaps=1;
 
-GLuint LoadTexture(SDL_Surface *surface, GLfloat *texcoord) {
-	GLuint texture;
-	int w, h;
-	SDL_Surface *image;
-	SDL_Rect area;
-	Uint32 saved_flags;
-	Uint8  saved_alpha;
+  /* Use the surface width and height expanded to powers of 2 */
+  w = powerOfTwo(surface->w);
+  h = powerOfTwo(surface->h);
+  
+  /* Rescale image if needed? */
+  GLint maxSize;
+  glGetIntegerv(GL_MAX_TEXTURE_SIZE,&maxSize);
+  double scale=1.0;
+  if(w > maxSize || h > maxSize)
+    scale=min(maxSize/(double)w,maxSize/(double)h);
+  
+  texcoord[0] = 0.0f;			/* Min X */
+  texcoord[1] = 0.0f;			/* Min Y */
+  texcoord[2] = (GLfloat)surface->w / w;	/* Max X */
+  texcoord[3] = (GLfloat)surface->h / h;	/* Max Y */
+  
+  image = SDL_CreateRGBSurface(
+			       SDL_SWSURFACE,
+			       (int)(w*scale), (int) (h*scale),
+			       32,
 
-	/* Use the surface width and height expanded to powers of 2 */
-	w = power_of_two(surface->w);
-	h = power_of_two(surface->h);
-
-	/* Rescale image if needed? */
-	GLint maxSize;
-	glGetIntegerv(GL_MAX_TEXTURE_SIZE,&maxSize);
-	double scale=1.0;
-	if(w > maxSize || h > maxSize)
-	  scale=min(maxSize/(double)w,maxSize/(double)h);
-
-	texcoord[0] = 0.0f;			/* Min X */
-	texcoord[1] = 0.0f;			/* Min Y */
-	texcoord[2] = (GLfloat)surface->w / w;	/* Max X */
-	texcoord[3] = (GLfloat)surface->h / h;	/* Max Y */
-
-	image = SDL_CreateRGBSurface(
-			SDL_SWSURFACE,
-			(int)(w*scale), (int) (h*scale),
-			32,
 #if SDL_BYTEORDER == SDL_LIL_ENDIAN /* OpenGL RGBA masks */
-			0x000000FF, 
-			0x0000FF00, 
-			0x00FF0000, 
-			0xFF000000
+			       0x000000FF, 
+			       0x0000FF00, 
+			       0x00FF0000, 
+			       0xFF000000
 #else
-			0xFF000000,
-			0x00FF0000, 
-			0x0000FF00, 
-			0x000000FF
+			       0xFF000000,
+			       0x00FF0000, 
+			       0x0000FF00, 
+			       0x000000FF
 #endif
-		       );
-	if ( image == NULL ) {
-		return 0;
-	}
+			       );
+  if ( image == NULL ) {
+    return 0;
+  }
+  
+  /* Save the alpha blending attributes */
+  saved_flags = surface->flags&(SDL_SRCALPHA|SDL_RLEACCELOK);
+  saved_alpha = surface->format->alpha;
+  if ( (saved_flags & SDL_SRCALPHA) == SDL_SRCALPHA ) {
+    SDL_SetAlpha(surface, 0, 0);
+  }
+  
+  /* Copy the surface into the GL texture image */
+  area.x = 0;
+  area.y = 0;
+  area.w = (int) (surface->w*scale);
+  area.h = (int) (surface->h*scale);
 
-	/* Save the alpha blending attributes */
-	saved_flags = surface->flags&(SDL_SRCALPHA|SDL_RLEACCELOK);
-	saved_alpha = surface->format->alpha;
-	if ( (saved_flags & SDL_SRCALPHA) == SDL_SRCALPHA ) {
-		SDL_SetAlpha(surface, 0, 0);
-	}
 
-	/* Copy the surface into the GL texture image */
-	area.x = 0;
-	area.y = 0;
-	area.w = (int) (surface->w*scale);
-	area.h = (int) (surface->h*scale);
-	if(scale == 1.0)
-	  // Easy, no scaling needed
-	  SDL_BlitSurface(surface, NULL, image, &area);
-	else {
-	  // Scaling needed
-	  // This is a realy inefficient and unoptimised way to do it. Sorry!
-	  static int hasWarned=0;
-	  if(!hasWarned) {
-		printf("Warning: Rescaling images before loading them as textures.\n");
-		hasWarned=1;
+  if(useMipmaps) scale=1.0;
+  
+  if(scale == 1.0)
+    // Easy, no scaling needed
+    SDL_BlitSurface(surface, NULL, image, &area);
+  else {
+    // Scaling needed
+    // This is a realy inefficient and unoptimised way to do it. Sorry!
+    static int hasWarned=0;
+    if(!hasWarned) {
+      printf(_("Warning: Rescaling images before loading them as textures.\n"));
+      hasWarned=1;
+    }
+    SDL_Surface *source=SDL_ConvertSurface(surface,image->format,SDL_SWSURFACE);
+    SDL_LockSurface(source);
+    SDL_LockSurface(image);
+    Uint32 *destPixels=(Uint32*)image->pixels;
+    Uint32 *sourcePixels=(Uint32*)source->pixels;
+    for(int x=0;x<area.w;x++)
+      for(int y=0;y<area.h;y++)
+	*((Uint32*)(((char*)image->pixels)+x*image->format->BytesPerPixel+y*image->pitch)) =
+	  *((Uint32*)(((char*)source->pixels)+((int)(x/scale))*source->format->BytesPerPixel+((int)(y/scale))*source->pitch));
+    SDL_UnlockSurface(source);
+    SDL_UnlockSurface(image);	   	  
+    SDL_FreeSurface(source);
+  }
+  
+  /* Restore the alpha blending attributes */
+  if ( (saved_flags & SDL_SRCALPHA) == SDL_SRCALPHA ) {
+    SDL_SetAlpha(surface, saved_flags, saved_alpha);
+  }
+  
+  /* Create an OpenGL texture for the image */       
+  GLuint freshTexture=0;
+  if(texture == NULL)
+    texture=&freshTexture;
+  if(*texture == 0) { 
+	glGenTextures(1, texture); //printf("texture: %d created\n",*texture); 
+  }
+  
+  glBindTexture(GL_TEXTURE_2D, *texture);
+	if( linearFilter ){
+	  if(useMipmaps) {
+		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR_MIPMAP_LINEAR);
+		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
+	  } else {
+		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+	  }
+
+	}else{
+	  if(useMipmaps) {
+		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST_MIPMAP_NEAREST);
+		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_NEAREST);
+	  } else {
+		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
+		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
 	  }
-	  SDL_Surface *source=SDL_ConvertSurface(surface,image->format,SDL_SWSURFACE);
-	  SDL_LockSurface(source);
-	  SDL_LockSurface(image);
-	  Uint32 *destPixels=(Uint32*)image->pixels;
-	  Uint32 *sourcePixels=(Uint32*)source->pixels;
-	  for(int x=0;x<area.w;x++)
-		for(int y=0;y<area.h;y++)
-		  *((Uint32*)(((char*)image->pixels)+x*image->format->BytesPerPixel+y*image->pitch)) =
-			*((Uint32*)(((char*)source->pixels)+((int)(x/scale))*source->format->BytesPerPixel+((int)(y/scale))*source->pitch));
-	  SDL_UnlockSurface(source);
-	  SDL_UnlockSurface(image);	   	  
-	  SDL_FreeSurface(source);
 	}
 
-	/* Restore the alpha blending attributes */
-	if ( (saved_flags & SDL_SRCALPHA) == SDL_SRCALPHA ) {
-		SDL_SetAlpha(surface, saved_flags, saved_alpha);
-	}
 
-	/* Create an OpenGL texture for the image */
-	glGenTextures(1, &texture);
-	glBindTexture(GL_TEXTURE_2D, texture);
-	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
-	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
-	glTexImage2D(GL_TEXTURE_2D,
-				 0,
-				 GL_RGBA,
-				 (int) (w*scale), (int) (h*scale),
-				 0,
-				 GL_RGBA,
-				 GL_UNSIGNED_BYTE,
-				 image->pixels);
-	SDL_FreeSurface(image); /* No longer needed */
-
-	return texture;
+	if(useMipmaps)
+	  gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGBA,(int)(w*scale),(int)(h*scale),GL_RGBA,GL_UNSIGNED_BYTE,image->pixels);
+	else
+	  glTexImage2D(GL_TEXTURE_2D,
+				   0,
+				   GL_RGBA,
+				   (int) (w*scale), (int) (h*scale),
+				   0,
+				   GL_RGBA,
+				   GL_UNSIGNED_BYTE,
+				   image->pixels);	
+	
+	SDL_FreeSurface(image); /* No longer needed */  
+	return *texture;
 }
+
 // YP: draw a spike with TRIANGLE_FAN. less time used by C->GL
 //     copies, and reduction of glBegin/glEnd number.
 void drawSpike(Coord3d a,Coord3d b,Coord3d c, Coord3d d) {
@@ -841,3 +1027,12 @@ void drawSpike(Coord3d a,Coord3d b,Coord3d c, Coord3d d) {
   glVertex3dv(c);
   glEnd();
 }
+
+
+
+void createForcefieldTextures() {
+}
+void resetForcefieldTextures() {
+}
+void bindForcefieldTexture(double t0) {
+}
