@@ -31,7 +31,7 @@
 #include "menuMode.h"
 #include "enterHighScoreMode.h"
 #include "sign.h"
-#include "SDL_image.h"
+#include "SDL/SDL_image.h"
 #include "forcefield.h"
 #include "scoreSign.h"
 #include "settings.h"
@@ -40,30 +40,25 @@
 
 using namespace std;
 
+#define ENVIRONMENT_TEXTURE_SIZE 128
+
 const int MainMode::statusBeforeGame=0,MainMode::statusGameOver=1,MainMode::statusInGame=2;
 const int MainMode::statusRestartPlayer=3,MainMode::statusNextLevel=4, MainMode::statusVictory=5;
 const int MainMode::statusLevelComplete=6,MainMode::statusBonusLevelComplete=7;
 const int MainMode::statusPaused=8;
-SDL_Surface *MainMode::panel,*MainMode::life,*MainMode::nolife;
+//SDL_Surface *MainMode::panel,*MainMode::life,*MainMode::nolife;
 
 MainMode *MainMode::mainMode=NULL;
+char *MainMode::viewportData, *MainMode::environmentTextureData;
 
 void MainMode::init() { 
   char str[256];
 
   mainMode = new MainMode(); 
 
-  sprintf(str,"%s/images/left_panel.png",SHARE_DIR);
-  panel = IMG_Load(str);
-  if(!panel) { printf("Error: Failed to load %s\n",str); exit(0); }  
-
-  sprintf(str,"%s/images/life.png",SHARE_DIR);
-  life = IMG_Load(str);
-  if(!life) { printf("Error: Failed to load %s\n",str); exit(0); }  
-
-  sprintf(str,"%s/images/nolife.png",SHARE_DIR);
-  nolife = IMG_Load(str);
-  if(!nolife) { printf("Error: Failed to load %s\n",str); exit(0); }  
+  /* Reflection maps */
+  viewportData=NULL;
+  environmentTextureData=(char*)malloc(sizeof(char)*4*ENVIRONMENT_TEXTURE_SIZE*ENVIRONMENT_TEXTURE_SIZE);
 }
 MainMode::MainMode() {
   this->go_to_pause = 0;
@@ -78,6 +73,7 @@ void MainMode::display() {
   Map *map = Game::current?Game::current->map:NULL;
   Player *player1 = Game::current?Game::current->player1:NULL;
   
+  glViewport(0,0,screenWidth,screenHeight);
   glPushAttrib(GL_ENABLE_BIT);
   
   if(Game::current->fogThickness)
@@ -96,7 +92,8 @@ void MainMode::display() {
 	glDisable(GL_FOG);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
-  gluPerspective(40,640.0/480.0,1.0,1e20);
+  gluPerspective(40,640.0/480.0,0.1,200); //40,640.0/480.0,1.0,1e20);
+
 
   /* Setup openGL matrixes for the camera perspective */
   glMatrixMode(GL_MODELVIEW);
@@ -116,26 +113,19 @@ void MainMode::display() {
 	gluLookAt(camFocus[0]-10.*sin(angle)*cos(zAngle*M_PI/2.), camFocus[1]-10.*cos(angle)*cos(zAngle*M_PI/2.), 
 			  10.0+camFocus[2]*0.5+(10.0+camFocus[2])*sin(zAngle*M_PI/2.),
 			  camFocus[0], camFocus[1], camFocus[2],
-			  up[0],up[1],up[2]); //0.0, 0.0, 1.0);
-	/*
-	gluLookAt(camFocus[0]-6.5, camFocus[1]-6.5, 10.0+camFocus[2]*0.5,
-			  camFocus[0], camFocus[1], camFocus[2],
-			  0.0, 0.0, 1.0);*/
+			  up[0],up[1],up[2]); 
   }
 
   /* Some standard GL settings needed */
   glEnable(GL_CULL_FACE);
   glEnable(GL_DEPTH_TEST);
   glDepthFunc(GL_LEQUAL);
+  glShadeModel(GL_SMOOTH);
 
-  GLfloat lightDiffuse[] = { 0.9, 0.9, 0.9, 1.0 };
-  GLfloat lightPosition[] = { -100.0, -50.0, 150.0, 0.0 };
-  glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse);
-  glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);
-  glEnable(GL_LIGHTING);
-  glEnable(GL_LIGHT0);
-  GLfloat ambient[] = { 0.2, 0.2, 0.2, 1.0 };
-  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient);
+  /* Debugging for problems with some cards */
+  glDisable(GL_TEXTURE_2D);
+  glDisable(GL_BLEND);
+  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
   if(Game::current && Game::current->isNight) {
     GLfloat lightDiffuse2[] = { 0.9, 0.9, 0.9, 1.0 };
@@ -150,9 +140,19 @@ void MainMode::display() {
     glLightfv(GL_LIGHT0, GL_SPECULAR, lightDiffuse2);
     glLightfv(GL_LIGHT0, GL_POSITION, lightPosition2);
     glLightf(GL_LIGHT0, GL_QUADRATIC_ATTENUATION, 0.25);
-  }
+	glEnable(GL_LIGHT0);
+  } else {
+	GLfloat sunLight[] = { 0.8, 0.8, 0.8, 1.0 };
+	GLfloat sunPosition[] = { -100.0, -50.0, 150.0, 0.0 };
+	glLightfv(GL_LIGHT0, GL_DIFFUSE, sunLight);
+	glLightfv(GL_LIGHT0, GL_POSITION, sunPosition);
+	glEnable(GL_LIGHT0);
 
-  glShadeModel(GL_SMOOTH);
+  }
+  GLfloat ambient[] = { 0.2, 0.2, 0.2, 1.0 };
+  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient);
+  glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL, GL_SEPARATE_SPECULAR_COLOR);
+  glEnable(GL_LIGHTING);
 
   /* Setup how we handle textures based on gfx_details */
   if(Settings::settings->gfx_details == 5) {
@@ -173,9 +173,9 @@ void MainMode::display() {
     if(map) map->draw(1,1,(int) map->startPosition[0],(int) map->startPosition[1]);
     { 
       char *lp[3], *rp[3];
-      lp[0] = "Track:"; rp[0] = Game::current->map->mapname;
-      lp[1] = "Author:"; rp[1] = Game::current->map->author;
-      lp[2] = "Press spacebar to begin"; rp[2] = " ";
+      lp[0] = _("Track:"); rp[0] = Game::current->map->mapname;
+      lp[1] = _("Author:"); rp[1] = Game::current->map->author;
+      lp[2] = _("Press spacebar to begin"); rp[2] = " ";
       multiMessage(3,lp,rp);
     }
     break;
@@ -189,10 +189,10 @@ void MainMode::display() {
 	  /*	  int buyCost;
 	  buyCost=(Game::current->player1->score/200)*100+500;
 	  if(Game::current->player1->score >= buyCost) 
-		sprintf(str,"Buy continue (%d points)?",buyCost);
+		snprintf(str,sizeof(str),"Buy continue (%d points)?",buyCost);
 		else*/
-		sprintf(str,"Press spacebar to continue");
-	  message("Game over",str);
+		snprintf(str,sizeof(str),_("Press spacebar to continue"));
+	  message(_("Game over"),str);
 	}
     break;
   case statusRestartPlayer:
@@ -200,13 +200,18 @@ void MainMode::display() {
     Game::current->draw();
     map->draw(birdsEye,1, (int) camFocus[0],(int) camFocus[1]);
     showInfo();
-    message("Oops","Press spacebar to continue");
+    message(_("Oops"),_("Press spacebar to continue"));
     break;
   case statusInGame:
+    //printf("before map draw 0\n"); SDL_GL_SwapBuffers(); SDL_GL_SwapBuffers(); getc(stdin); 
     map->draw(birdsEye,0, (int) camFocus[0],(int) camFocus[1]);
+    //printf("before game draw\n"); SDL_GL_SwapBuffers(); SDL_GL_SwapBuffers(); getc(stdin);
     Game::current->draw();
+    //printf("before map draw 1\n"); SDL_GL_SwapBuffers(); SDL_GL_SwapBuffers(); getc(stdin); 
     map->draw(birdsEye,1, (int) camFocus[0],(int) camFocus[1]);
+    //printf("before showinfo\n"); SDL_GL_SwapBuffers(); SDL_GL_SwapBuffers(); getc(stdin); 
     showInfo();
+    //printf("after everything\n"); SDL_GL_SwapBuffers(); SDL_GL_SwapBuffers(); getc(stdin); 
     break;
   case statusPaused:
     map->draw(birdsEye,0, (int) camFocus[0],(int) camFocus[1]);
@@ -228,16 +233,14 @@ void MainMode::display() {
     Leave2DMode();
     // print 'paused'
     glColor4f(1., 1., 1., 1.);
-	Font::drawCenterSimpleText(0,"Paused",screenWidth/2,screenHeight/2-16,16.,16.,1.0,1.0,1.0,0.75);
-    //draw2DString(menuFont, "Paused", screenWidth/2-32, screenHeight/2-10,
-	//               255,255,255);
+	Font::drawCenterSimpleText(0,_("Paused"),screenWidth/2,screenHeight/2-16,16.,16.,1.0,1.0,1.0,0.75);
     break;
   case statusBonusLevelComplete:
 	map->draw(birdsEye,0, (int) camFocus[0],(int) camFocus[1]);
 	Game::current->draw();
 	map->draw(birdsEye,1, (int) camFocus[0],(int) camFocus[1]);
 	showInfo();
-	message("Bonus level complete","Press spacebar to continue");
+	message(_("Bonus level complete"),_("Press spacebar to continue"));
 	break;
   case statusLevelComplete:
 	map->draw(birdsEye,0, (int) camFocus[0],(int) camFocus[1]);
@@ -251,14 +254,14 @@ void MainMode::display() {
 	Game::current->draw();
 	map->draw(birdsEye,1, (int) camFocus[0],(int) camFocus[1]);
 	showInfo();
-	message("Level complete","Press spacebar to continue");
+	message(_("Level complete"),_("Press spacebar to continue"));
 	break;
   case statusVictory:
 	map->draw(birdsEye,0, (int) camFocus[0],(int) camFocus[1]);
 	Game::current->draw();
 	map->draw(birdsEye,1, (int) camFocus[0],(int) camFocus[1]);
 	showInfo();
-	message("Congratulations","You have completed the game!");
+	message(_("Congratulations"),_("You have completed the game!"));
 	break;
   }
 
@@ -280,8 +283,28 @@ void MainMode::display() {
 	Font::drawCenterSimpleText(0,str,screenWidth/2,screenHeight-16, 8., 8., 0.6, 0.6, 0.6, 0.6);
 	Leave2DMode();
   }
+
+  /* For debugging, this draws the players environment map on the upper right corner of the screen */
+  /*
+  Enter2DMode();
+  glEnable(GL_TEXTURE_2D);
+  glBindTexture(GL_TEXTURE_2D,Game::current->player1->environmentTexture);
+  glColor4f(1., 1., 1., 1.0);
+  glEnable(GL_BLEND);
+  glBegin(GL_TRIANGLE_STRIP);
+  glTexCoord2f(1.0,1.0); glVertex2i(screenWidth-0-10, 0);
+  glTexCoord2f(0.0,1.0); glVertex2i(screenWidth-ENVIRONMENT_TEXTURE_SIZE-10, 0);
+  glTexCoord2f(1.0,0.0); glVertex2i(screenWidth-0-10, ENVIRONMENT_TEXTURE_SIZE);
+  glTexCoord2f(0.0,0.0); glVertex2i(screenWidth-ENVIRONMENT_TEXTURE_SIZE-10, ENVIRONMENT_TEXTURE_SIZE);
+  glEnd();
+  Leave2DMode();
+  */
+
   glPopAttrib();
 }
+void MainMode::doExpensiveComputations() { Game::current->doExpensiveComputations(); }
+
+
 void MainMode::key(int key) {  
   Player *player1 = Game::current?Game::current->player1:NULL;
   
@@ -301,18 +324,16 @@ void MainMode::key(int key) {
     //gameStatus = statusInGame;
     break;
   case statusInGame:
-        if ((key == 'p')||(key == 'P'))
-          {
+	if ((key == 'p')||(key == 'P')) {
 	  go_to_pause = 1;
 	  pause_time = 0.;
-          gameStatus = statusPaused;
-          break;
-          }
-	if (key == SDLK_PRINT)
-	  {
+	  gameStatus = statusPaused;
+	  break;
+	}
+	if (key == SDLK_PRINT) {
 	  createSnapshot();
 	  break;
-	  }
+	}
 	player1->key(key); 
 	break;
   case statusBonusLevelComplete:
@@ -359,14 +380,11 @@ void MainMode::idle(Real td) {
   double t;
   int i;
   
+
   time += td;
   flash -= td;
   if(flash < 0.0) flash = 0.0;
 
-  if(Settings::settings->hasJoystick()) {
-	if(Settings::settings->joystickButton(0)) key(' ');
-  }
-
   switch(gameStatus) {
   case statusBeforeGame: 
     break;
@@ -446,8 +464,13 @@ void MainMode::activated() {
   camFocus[1] = Game::current->map->startPosition[1] - 5;
   time = 0.0;
   flash = 0.0;
+
+  /* Fix for an apparanet bug in my SDL + Xorg combination... */
+  SDL_WM_ToggleFullScreen(screen);
+  SDL_WM_ToggleFullScreen(screen);
+
 }
-void MainMode::deactivated() {  }
+void MainMode::deactivated() { free(viewportData); viewportData=NULL; }
 void MainMode::playerLoose() { Game::current->gamer->playerLoose(); gameStatus=statusGameOver; }
 void MainMode::playerDie() { gameStatus=statusRestartPlayer; }
 
@@ -455,8 +478,9 @@ void MainMode::playerDie() { gameStatus=statusRestartPlayer; }
 void MainMode::mouse(int state,int x,int y) { 
   if(Game::current) Game::current->player1->mouse(state,x,y); 
 }
-void MainMode::mouseDown(int button,int x,int y) { key(' '); } // if(button && gameStatus != statusInGame) key(' '); }
-
+void MainMode::mouseDown(int button,int x,int y) { 
+  if(button == SDL_BUTTON_LMASK) key(' '); 
+}
 /* Starts the current game */
 void MainMode::startGame() {
   gameStatus = statusInGame;
@@ -465,7 +489,7 @@ void MainMode::startGame() {
   Coord3d pos;
   assign(Game::current->map->startPosition,pos);
   pos[2] += 2.0;
-  new Sign("Good luck!",7,1.0,60.0,pos);
+  new Sign(_("Good luck!"),6,1.0,60.0,pos);
   Game::current->player1->position[2] += 1.0;
 }
 
@@ -480,7 +504,7 @@ void MainMode::restartPlayer() {
   Coord3d pos;
   assign(Game::current->map->startPosition,pos);
   pos[2] += 2.0;
-  new Sign("Good luck!",7,1.0,60.0,pos);
+  new Sign(_("Good luck!"),7,1.0,60.0,pos);
 }
 
 /* Shows various information on screen */
@@ -492,32 +516,31 @@ void MainMode::showInfo() {
 
   glColor3f(1.0,1.0,1.0);
 
+
+  Enter2DMode();
+
   // the panel
-  drawSurface(panel,0,0,panel->w,panel->h);
+  bindTexture("left_panel.png");
+  drawTextured2DRectangle(0,0,256,128);  
 
   // lives
-  for(i=0;i<4;i++)
-	drawSurface(i<player->lives?life:nolife,63+i*20,7,life->w,life->h);
+  for(i=0;i<4;i++) {
+	if(i<player->lives) bindTexture("life.png");
+	else bindTexture("nolife.png");
+	drawTextured2DRectangle(63+i*20,7,32,32);
+  }
 
   // Score
-  sprintf(str,"%d",player->score);
-  //TTF_SizeText(infoFont,str,&w,&h);
-  //draw2DString(infoFont,str,148-w,65,0,0,0);
+  snprintf(str,sizeof(str),"%d",player->score);
   Font::drawRightSimpleText(0,str,152,52,8.,8.,0.0,0.0,0.0,1.0);
 
   // Time left
-  sprintf(str,"%2.2d:%2.2d",player->timeLeft/60,player->timeLeft%60);
-  //  TTF_SizeText(infoFont,str,&w,&h);
+  snprintf(str,sizeof(str),"%2.2d:%2.2d",player->timeLeft/60,player->timeLeft%60);
   if(player->timeLeft < 15 && player->timeLeft % 2)
-	//draw2DString(infoFont,str,148-w,94,255,255,255);
 	Font::drawRightSimpleText(0,str,152,81,8.,8.,1.0,1.0,1.0,1.0);
   else
-	//draw2DString(infoFont,str,148-w,94,0,0,0);
 	Font::drawRightSimpleText(0,str,152,81,8.,8.,0.0,0.0,0.0,1.0);
-  
-	//	draw2DString(infoFont,str,10,20+1*18,255,255,64); 
 
-  Enter2DMode(); 
 
   // Health
   glDisable(GL_TEXTURE_2D);
@@ -548,60 +571,9 @@ void MainMode::showInfo() {
   glVertex2i(8, 8);  
   glEnd(),
 
+
   Leave2DMode();
-
-  /*
-  for(i=0;i<3;i++)
-	drawSurface(player->modTimeLeft[i]?hasmod:nomod,(screenWidth-panel->w)/2+320,5+24*i,hasmod->w,hasmod->h);
-  for(i=0;i<3;i++)
-	drawSurface(player->modTimeLeft[i+3]?hasmod:nomod,(screenWidth-panel->w)/2+447,5+24*i,hasmod->w,hasmod->h);
-  */
-
-  /*
-  int nMods=0;
-  if(player->modTimeLeft[MOD_SPEED]) {
-	sprintf(str,"Speedball %d sec",(int)player->modTimeLeft[MOD_SPEED]);
-	draw2DString(infoFont,str,screenWidth-120,20+nMods*18,64,64,255); 
-	nMods++;
-  }
-  if(player->modTimeLeft[MOD_JUMP]) {
-	sprintf(str,"Jumpjets %d sec",(int)player->modTimeLeft[MOD_JUMP]);
-	draw2DString(infoFont,str,screenWidth-120,20+nMods*18,64,64,255); 
-	nMods++;
-  }
-  if(player->modTimeLeft[MOD_SPIKE]) {
-	sprintf(str,"Spikes %d sec",(int)player->modTimeLeft[MOD_SPIKE]);
-	draw2DString(infoFont,str,screenWidth-120,20+nMods*18,64,64,255); 
-	nMods++;
-  }
-  if(player->modTimeLeft[MOD_GLASS]) {
-	sprintf(str,"Glassball %d sec",(int)player->modTimeLeft[MOD_GLASS]);
-	draw2DString(infoFont,str,screenWidth-120,20+nMods*18,255,64,64); 
-	nMods++;
-  }
-  if(player->modTimeLeft[MOD_DIZZY]) {
-	sprintf(str,"Dizzy %d sec",(int)player->modTimeLeft[MOD_DIZZY]);
-	draw2DString(infoFont,str,screenWidth-120,20+nMods*18,255,64,64); 
-	nMods++;
-  }
-  if(player->modTimeLeft[MOD_FROZEN]) {
-	sprintf(str,"Frozen %d sec",(int)player->modTimeLeft[MOD_FROZEN]);
-	draw2DString(infoFont,str,screenWidth-120,20+nMods*18,255,64,64); 
-	nMods++;
-	}*/
-
-
-  /*
-  sprintf(str,"In the air: %d",player->inTheAir);
-  draw2DString(infoFont,str,screenWidth-120,20,64,255,255); 
-  sprintf(str,"velocity[2]=%f",player->velocity[2]);
-  draw2DString(infoFont,str,screenWidth-120,20+1*18,64,255,255); 
-  sprintf(str,"gametime=%f",Game::current->gameTime);
-  draw2DString(infoFont,str,screenWidth-120,20+2*18,64,255,255); 
-  */
-
-
-
+  return;
 }
 void MainMode::levelComplete() {
   Game::current->player1->hasWon = 1;
@@ -621,11 +593,11 @@ void MainMode::showBonus() {
 	right_pointers[i] = &right[i][0];
   }
 
-  sprintf(left[0],"Level bonuses"); sprintf(right[0]," ");
-  sprintf(left[1],"Lives left:"); sprintf(right[1],"%d x 100 = %d points",player->lives,player->lives*100);
-  sprintf(left[2],"Time left:"); sprintf(right[2],"%d minutes = %d points",player->timeLeft/60,(player->timeLeft/60)*250);
-  sprintf(left[3],"Difficulty:");
-  sprintf(right[3],"%s = %d points",difficulty == 0 ? "easy" : (difficulty == 1 ? "normal" : "hard"),difficulty * 500);
+  snprintf(left[0],255,_("Level bonuses")); snprintf(right[0],255," ");
+  snprintf(left[1],255,_("Lives left:")); snprintf(right[1],255,_("%d x 100 = %d points"),player->lives,player->lives*100);
+  snprintf(left[2],255,_("Time left:")); snprintf(right[2],255,_("%d minutes = %d points"),player->timeLeft/60,(player->timeLeft/60)*250);
+  snprintf(left[3],255,_("Difficulty:"));
+  snprintf(right[3],255,"%s = %d points",difficulty == 0 ? _("easy") : (difficulty == 1 ? _("normal") : _("hard")),difficulty * 500);
 
   multiMessage(min((int)statusCount+1,4),left_pointers,right_pointers);
 }
@@ -633,3 +605,199 @@ void MainMode::bonusLevelComplete() {
   gameStatus = statusBonusLevelComplete;
   strcpy(Game::current->nextLevel,Game::current->returnLevel);
 }
+
+
+void MainMode::renderEnvironmentTexture(GLuint texture,Coord3d focus) {
+  // Save gfx_details and change before drawing environment */
+  int gfx_details=Settings::settings->gfx_details;
+  Settings::settings->gfx_details = GFX_DETAILS_MINIMALISTIC;
+
+  static int currentViewportSize=512;
+  static double currentTime=0.0;
+  if(currentTime > 0.05) currentViewportSize = max(128,currentViewportSize / 2);
+  else if(currentTime < 0.05/4.0) currentViewportSize = min(512,currentViewportSize*2);
+
+  int viewportSize=currentViewportSize;
+  if(screenHeight < 512) viewportSize=screenHeight;
+  //viewportSize=256;
+
+  double t0 = ((double)SDL_GetTicks())/1000.0;
+
+  glViewport(0,0,viewportSize,viewportSize);
+  if(Game::current->fogThickness)
+	glClearColor(Game::current->fogColor[0],Game::current->fogColor[1],Game::current->fogColor[2],Game::current->fogColor[3]);
+  else
+	glClearColor(0.0,0.0,0.0,0.0);
+  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
+
+  glPushAttrib(GL_ENABLE_BIT);
+  if(Game::current->fogThickness && Settings::settings->gfx_details != GFX_DETAILS_NONE) {
+	glEnable(GL_FOG);
+	glFogi(GL_FOG_MODE,GL_LINEAR);
+	glFogf(GL_FOG_START,max(0.0,14.0-7.0*Game::current->fogThickness));
+	glFogf(GL_FOG_END,26.0-4.0*Game::current->fogThickness);
+	glFogfv(GL_FOG_COLOR,Game::current->fogColor);
+  } else
+	glDisable(GL_FOG);
+  glMatrixMode(GL_PROJECTION);
+  glLoadIdentity();
+  gluPerspective(140,1.0,0.01,200);
+
+  /* Setup openGL matrixes for the camera perspective */
+  glMatrixMode(GL_MODELVIEW);
+  glLoadIdentity();
+  int birdsEye=1;
+  double angle=xyAngle*M_PI/2. + M_PI/4.;
+  // TODO. Fixme. This computation is wrong when zAngle > 0.0 !!
+  Coord3d up; up[0]=sin(angle)*zAngle; up[1]=cos(angle)*zAngle; up[2]=1.0-zAngle; normalize(up);
+  gluLookAt(focus[0], focus[1], focus[2]+0.0,
+			focus[0]-10.*sin(angle)*cos(zAngle*M_PI/2.), 
+			focus[1]-10.*cos(angle)*cos(zAngle*M_PI/2.), 
+			zAngle>0.0 ? 
+			10.0+focus[2]*0.5+(10.0+focus[2])*sin(zAngle*M_PI/2.) :
+			focus[2]+2.0
+			, 
+			up[0],up[1],up[2]);
+
+  /* Some standard GL settings needed */
+  glEnable(GL_CULL_FACE);
+  glEnable(GL_DEPTH_TEST);
+  glDepthFunc(GL_LEQUAL);
+
+  GLfloat lightDiffuse[] = { 0.9, 0.9, 0.9, 1.0 };
+  GLfloat lightPosition[] = { -100.0, -50.0, 150.0, 0.0 };
+  glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse);
+  glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);
+  glEnable(GL_LIGHTING);
+  glEnable(GL_LIGHT0);
+  GLfloat ambient[] = { 0.2, 0.2, 0.2, 1.0 };
+  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient);
+
+  if(Game::current && Game::current->isNight) {
+    GLfloat lightDiffuse2[] = { 0.9, 0.9, 0.9, 1.0 };
+    GLfloat lightPosition2[4] = {252.0,252.0,10.0,1.0};
+    lightPosition2[0] = Game::current->player1->position[0];
+    lightPosition2[1] = Game::current->player1->position[1];
+    lightPosition2[2] = Game::current->player1->position[2] + 2.0;
+    GLfloat black[] = { 0.0,0.0,0.0,1.0 };
+    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, black);
+    glLightfv(GL_LIGHT0, GL_AMBIENT, black);
+    glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse2);
+    glLightfv(GL_LIGHT0, GL_SPECULAR, lightDiffuse2);
+    glLightfv(GL_LIGHT0, GL_POSITION, lightPosition2);
+    glLightf(GL_LIGHT0, GL_QUADRATIC_ATTENUATION, 0.25);
+  }
+
+  glShadeModel(GL_SMOOTH);
+
+  /* Setup how we handle textures based on gfx_details */
+  if(Settings::settings->gfx_details == 5) {
+	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
+	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
+	glHint(GL_FOG_HINT,GL_NICEST);
+  } else {
+	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
+	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
+  	glHint(GL_FOG_HINT,GL_FASTEST);
+  }
+
+  Map *map = Game::current?Game::current->map:NULL;
+  Player *player1 = Game::current?Game::current->player1:NULL;
+
+  map->draw(birdsEye,0, (int) focus[0]+10,(int) focus[1]+10);
+  Game::current->drawReflection(focus);
+  //map->draw(birdsEye,1, (int) focus[0]+10,(int) focus[1]+10);
+
+  glPopAttrib();
+
+  /* Copy rendered image into viewportData */
+  if(!viewportData) {
+	viewportData=(char*)malloc(sizeof(char)*screenWidth*screenHeight*3); 
+  }
+  // approx 1-2 ms */
+  glReadPixels(0, 0, viewportSize, viewportSize, GL_RGB, GL_UNSIGNED_BYTE, (GLvoid*)viewportData);
+
+  // Converts viewport data to fisheye environment perspective
+  // approx 10 ms */
+  double t0_5 = ((double)SDL_GetTicks())/1000.0;
+  convertToFisheye(environmentTextureData,viewportData,viewportSize);
+
+  /* Takes approx 1-2 ms */
+  glBindTexture(GL_TEXTURE_2D, texture);
+  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+  glTexImage2D(GL_TEXTURE_2D,
+			   0,
+			   GL_RGBA, // GL_RGBA
+			   ENVIRONMENT_TEXTURE_SIZE, ENVIRONMENT_TEXTURE_SIZE,
+			   0,
+			   GL_RGBA,
+			   GL_UNSIGNED_BYTE,
+			   environmentTextureData);
+  double t1 = ((double)SDL_GetTicks())/1000.0;
+  currentTime=t1-t0;
+  //printf("SIZE: %d, TD: %f + %f = %f\n",viewportSize,t0_5-t0,t1-t0_5,t1-t0);
+
+  // Restore graphics details
+  Settings::settings->gfx_details = gfx_details;
+}
+
+/* Creates a 128x128 fisheye texture from viewport data */
+void MainMode::convertToFisheye(char *data,char *viewport,int viewportSize) {
+
+  /* TODO. Let the environment map sample multiple viewport pixels.
+	 Encode an alpha value directly in the translationmap and let the alpha value increase
+	 as the angle away from the observer increases. 
+  */
+  int x,y,i,j,x2,y2;
+  static short *translationMap=NULL;
+  static short translationViewportSize=0;
+  if(!translationMap) translationMap=(short*) malloc(sizeof(short)*ENVIRONMENT_TEXTURE_SIZE*ENVIRONMENT_TEXTURE_SIZE*2);
+  if(translationViewportSize != viewportSize) {
+	translationViewportSize=viewportSize;
+	/* Create mapping between fisheye texture and viewport */
+	int x,y,i,x2,y2;
+	for(x=0;x<ENVIRONMENT_TEXTURE_SIZE;x++) {
+	  double nx=(x-ENVIRONMENT_TEXTURE_SIZE/2)*2.0/ENVIRONMENT_TEXTURE_SIZE;
+	  for(y=0;y<ENVIRONMENT_TEXTURE_SIZE;y++) {
+		double ny=(y-ENVIRONMENT_TEXTURE_SIZE/2)*2.0/ENVIRONMENT_TEXTURE_SIZE;
+		double nz=1.0-nx*nx-ny*ny;
+		if(nz>0.0) {
+		  nz=sqrt(nz);
+		  double dotprod=2.0*nz;
+		  double vx=-2.0*nx;
+		  double vy=-2.0*ny;
+		  double vz=1-2.0*nz;
+		  double h=0.05 / vz;
+		  double fx2=-h * vx;
+		  double fy2=+h * vy;
+		  if(dotprod<1.0) { fy2=-fy2; fx2=-fx2; }
+		  if(fx2>0.5) { fx2=0.5; fy2=(fy2/fx2)*0.5; }
+		  else if(fx2<-0.5) { fx2=-0.5; fy2=(fy2/-fx2)*0.5; }
+		  if(fy2>0.5) { fy2=0.5; fx2=(fx2/fy2)*0.5; }
+		  else if(fy2<-0.5) { fy2=-0.5; fx2=(fx2/-fy2)*0.5; }
+		  x2=(int) (viewportSize*(fx2*0.99+0.5));
+		  y2=(int) (viewportSize*(fy2*0.99+0.5));
+		  
+		  translationMap[(x+y*ENVIRONMENT_TEXTURE_SIZE)*2] = x2;
+		  translationMap[(x+y*ENVIRONMENT_TEXTURE_SIZE)*2+1] = y2;
+		} else {
+		  translationMap[(x+y*ENVIRONMENT_TEXTURE_SIZE)*2+0] = -1;
+		  translationMap[(x+y*ENVIRONMENT_TEXTURE_SIZE)*2+1] = 0;
+		}
+	  }
+	}
+  }
+
+  int size=ENVIRONMENT_TEXTURE_SIZE*ENVIRONMENT_TEXTURE_SIZE;
+  for(i=0;i<size;i++) {
+	x2=translationMap[i*2+0];
+	y2=translationMap[i*2+1];
+	if(x2 == -1) data[i*4+3]=0;
+	else {
+	  char *viewportPntr=&viewport[(x2+y2*viewportSize)*3];
+	  for(j=0;j<3;j++) data[i*4+j] = viewportPntr[j];
+	  data[i*4+3]=255;
+	}
+  }
+}
