@@ -35,16 +35,6 @@ class MenuMode : public GameMode {
   static MenuMode *menuMode;
  private:
   void doSelection();
-
-  int selection;
-  static char *menuNames[];
-  static SDL_Surface *menuLow[], *menuHigh[]; 
-  static SDL_Surface *header;
-
-  GLuint texture;
-  GLfloat texMinX, texMinY;
-  GLfloat texMaxX, texMaxY;
-  GLfloat texDY;
   double offset;
 };
 
