@@ -50,7 +50,11 @@ char *SHARE_DIR_DEFAULT=SHARE_DIR;
 #include "helpMode.h"
 #include "map.h"
 #include "calibrateJoystickMode.h"
-
+#ifdef WIN32
+#define WIN32_LEAN_AND_MEAN 1
+#include <windows.h>
+#include <shlobj.h>
+#endif
 using namespace std;
 
 /* Important globals */
@@ -60,13 +64,13 @@ int silent=0;
 int debug_joystick,repair_joystick;
 
 char effectiveShareDir[256];
-int screenResolutions[4][2] = {{640,480}, {800,600}, {1024,768}, {1280,1024}}, nScreenResolutions=4;
+int screenResolutions[5][2] = {{640,480}, {800,600}, {1024,768}, {1280,1024}, {1600,1200}}, nScreenResolutions=5;
 
 void changeScreenResolution() {
   screenWidth = screenResolutions[Settings::settings->resolution][0]; 
   screenHeight = screenResolutions[Settings::settings->resolution][1];
 
-  if(Settings::settings->colorDepth == 16 || 1) {
+  if(Settings::settings->colorDepth == 16) {
 	SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 5);
 	SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 5);
 	SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 5);
@@ -79,40 +83,48 @@ void changeScreenResolution() {
   SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER,1);
 
   screen = SDL_SetVideoMode(screenResolutions[Settings::settings->resolution][0], 
-							screenResolutions[Settings::settings->resolution][1], 
-							Settings::settings->colorDepth, 
-							SDL_SWSURFACE | SDL_OPENGL | (Settings::settings->is_windowed?0:SDL_FULLSCREEN));
+			    screenResolutions[Settings::settings->resolution][1], 
+			    Settings::settings->colorDepth, 
+			    SDL_SWSURFACE | SDL_OPENGL | (Settings::settings->is_windowed?0:SDL_FULLSCREEN));
+  if(!screen) return;
   screenWidth = screen->w; screenHeight = screen->h;
   SDL_WarpMouse(screenWidth/2,screenHeight/2);	
+  SDL_WM_GrabInput(SDL_GRAB_ON);
+
+  resetTextures();
 }
 
 void print_usage (FILE* stream, int exit_code) {
-       fprintf (stream, "Usage: %s [-w, -m] [-e, -l -t <level>] [-r <width>] [-s <sensitivity>]\n", program_name);
-       fprintf (stream, "   -h  --help         Display this usage information.\n"
-				"   -e  --edit         Start as level editor.\n"
-				"   -l  --level        Start from level.\n"
-				"   -w  --windowed     Run in window (Default is fullscreen)\n"
-				"   -m  --mute         Mute sound.\n"
-				"   -r  --resolution   Set resolution to 640, 800 or 1024\n"
-				"   -s  --sensitivity  Mouse sensitivity, default 1.0\n"
-				"   -f  --fps          Displays framerate\n"
-				"   -q  --quiet        Do not print anything to stdout\n"
-				"   -v  --version      Prints current version number\n"
-				"   -t  --touch        Updates a map to the latest format\n"
-				"   -y  --low-memory   Attempt to conserve memory usage\n"
-				"   -j  --repair-joystick Correct for bad joysticks\n"
-			);
-       exit (exit_code);
+  fprintf (stream, _("Usage: %s [-w, -m] [-e, -l -t <level>] [-r <width>] [-s <sensitivity>]\n"), program_name);
+  fprintf (stream, _("   -h  --help            Display this usage information.\n"));
+  fprintf (stream, _("   -e  --edit            Start as level editor.\n"));
+  fprintf (stream, _("   -l  --level           Start from level.\n"));
+  fprintf (stream, _("   -w  --windowed        Run in window (Default is fullscreen)\n"));
+  fprintf (stream, _("   -m  --mute            Mute sound.\n"));
+  fprintf (stream, _("   -r  --resolution      Set resolution to 640, 800 or 1024\n"));
+  fprintf (stream, _("   -s  --sensitivity     Mouse sensitivity, default 1.0\n"));
+  fprintf (stream, _("   -f  --fps             Displays framerate\n"));
+  fprintf (stream, _("   -q  --quiet           Do not print anything to stdout\n"));
+  fprintf (stream, _("   -v  --version         Prints current version number\n"));
+  fprintf (stream, _("   -t  --touch           Updates a map to the latest format\n"));
+  fprintf (stream, _("   -y  --low-memory      Attempt to conserve memory usage\n"));
+  fprintf (stream, _("   -j  --repair-joystick Correct for bad joysticks\n"));
+  exit (exit_code);
 }
 
 int testDir() {
   if(!strlen(effectiveShareDir)) return 0;
+#ifdef WIN32
+  // Really this is just done for asthetics
+  for(int i=strlen(effectiveShareDir)-1;i>=0;i--)
+    if( effectiveShareDir[i] == '\\' ) effectiveShareDir[i] = '/';
+#endif
   DIR *dir = opendir(effectiveShareDir);
   //printf("Looking for %s\n", effectiveShareDir);
   if(!dir) return 0;
   else closedir(dir);
   char str[256];
-  sprintf(str,"%s/levels",effectiveShareDir);
+  snprintf(str,sizeof(str),"%s/levels",effectiveShareDir);
   dir=opendir(str);
   if(!dir) return 0;
   else closedir(dir);
@@ -153,20 +165,36 @@ void innerMain(void *closure,int argc,char **argv) {
   Settings::init();
   Settings *settings = Settings::settings;
   settings->doSpecialLevel=0;
+  settings->setLocale();                 /* Start "correct" i18n as soon as possible */
   low_memory=0; debug_joystick=0; repair_joystick=0;
   do {
+
+#if defined (__SVR4) && defined (__sun)
+	next_option = getopt(argc,argv,short_options);
+#else
+	next_option = getopt_long (argc, argv, short_options, 
+	long_options, NULL);
+#endif
+
+	/*
 #ifdef solaris
 	next_option = getopt(argc,argv,short_options);
 #else
 	next_option = getopt_long (argc, argv, short_options, 
 	long_options, NULL);
 #endif
+	*/
+
 	switch (next_option) {
 	case 'h':
 	  print_usage (stdout, 0);
 	case 'e':
-	  editMode = 1;
-	  sprintf(Settings::settings->specialLevel,"%s",optarg);
+	  fprintf(stderr,_("Commandline switch -e is deprecated, use the editor button the main menu instead\n"));
+	  break;
+	  //editMode = 1;
+	  // fall through to l
+	case 'l':
+	  snprintf(Settings::settings->specialLevel,sizeof(Settings::settings->specialLevel)-1,"%s",optarg);
 	  Settings::settings->doSpecialLevel=1;
 	  break;
 	case 't':
@@ -174,10 +202,6 @@ void innerMain(void *closure,int argc,char **argv) {
 	  touchName=optarg;
 	  audio=0; // no audio
 	  break;
-	case 'l':
-	  sprintf(Settings::settings->specialLevel,"%s",optarg);
-	  Settings::settings->doSpecialLevel=1;
-	  break;
 	case 'w': settings->is_windowed=1;  break;
 	case 'm':
 	  audio = 0;
@@ -185,7 +209,7 @@ void innerMain(void *closure,int argc,char **argv) {
 	case 'r':
 	  for(i=0;i<nScreenResolutions;i++) if(screenResolutions[i][0] == atoi(optarg)) break;
 	  if(i<nScreenResolutions) settings->resolution=i;
-	  else { printf("Unknown screen resolution of width %d\n",i); }
+	  else { printf(_("Unknown screen resolution of width %d\n"),i); }
 	  break;
 	case 's':
 	  Settings::settings->mouseSensitivity = atof(optarg);
@@ -210,23 +234,22 @@ void innerMain(void *closure,int argc,char **argv) {
   while (next_option != -1);
   
   if(!silent) {
-	printf("Welcome to Trackballs. \n");
-	printf("Using %s as gamedata dir\n",SHARE_DIR);
+	printf(_("Welcome to Trackballs. \n"));
+	printf(_("Using %s as gamedata directory.\n"),SHARE_DIR);
   }
 
   /* Initialize SDL */
   if((SDL_Init(SDL_INIT_VIDEO|audio|SDL_INIT_JOYSTICK)==-1)) {
-	printf("failed!\n"); fflush(stdout);
-	printf("Could not initialize SDL: %s.\n", SDL_GetError());
+	printf(_("Could not initialize libSDL.\nError message: '%s'\n"), SDL_GetError());
 	exit(-1);
   }  
   atexit(SDL_Quit);
 
   char buffer[1024];
-  sprintf(buffer, "/%s/ V%s", PACKAGE, VERSION);
+  snprintf(buffer,sizeof(buffer),"/%s/ V%s", PACKAGE, VERSION);
   SDL_WM_SetCaption(buffer, buffer);
 
-  sprintf(str,"%s/icons/trackballs-32x32.png",SHARE_DIR);
+  snprintf(str,sizeof(str),"%s/icons/trackballs-32x32.png",SHARE_DIR);
   SDL_Surface *wmIcon = IMG_Load(str);
   if(wmIcon) {  SDL_WM_SetIcon(wmIcon, NULL); }
 
@@ -238,7 +261,7 @@ void innerMain(void *closure,int argc,char **argv) {
   // set the name of the window
 
   double bootStart=((double)SDL_GetTicks()) /1000.0 ;
-  sprintf(str,"%s/images/splashScreen.jpg",SHARE_DIR);
+  snprintf(str,sizeof(str),"%s/images/splashScreen.jpg",SHARE_DIR);
   SDL_Surface *splashScreen = IMG_Load(str);
   if(!splashScreen) { printf("Error: Failed to load %s\n",str); exit(0); }
   glViewport(0,0,screenWidth,screenHeight);
@@ -270,29 +293,31 @@ void innerMain(void *closure,int argc,char **argv) {
 	
 	snprintf(mapname,sizeof(mapname)-1,"%s/.trackballs/levels/%s.map",getenv("HOME"),touchName);
 	if(!fileExists(mapname))
-	  sprintf(mapname,"%s/levels/%s.map",SHARE_DIR,touchName);
+	  snprintf(mapname,sizeof(mapname),"%s/levels/%s.map",SHARE_DIR,touchName);
 	if(!fileExists(mapname))
-	  snprintf(mapname,511,"%s",touchName);
+	  snprintf(mapname,sizeof(mapname),"%s",touchName);
 	printf("Touching map %s\n",mapname);
 	Map *map = new Map(mapname);
 	map->save(mapname,(int) map->startPosition[0],(int) map->startPosition[1]);
 	exit(0);
   } else if(editMode) {
+	EditMode::init();
 	Font::init();
-    GameMode::activate(new EditMode());
+	GameMode::activate(EditMode::editMode);
   } else {
 
 	// MB: Reminder 51 megs it was
-    	Font::init();	       // MB: Until here we are using 54 megs of memory. 
+	Font::init();	       // MB: Until here we are using 54 megs of memory. 
 	MenuMode::init();      // MB: Until here we are using 55 megs of memory. 
 
 	MainMode::init();
 	HighScore::init();     // MB: 55
+	EditMode::init();
 
 	EnterHighScoreMode::init(); // MB: 58
 	HallOfFameMode::init();     // MB: 62
 	SettingsMode::init();       // MB: 64	
-    GameMode::activate(MenuMode::menuMode);	// MB: 65
+	GameMode::activate(MenuMode::menuMode);	// MB: 65
 	GameHook::init();
 	CalibrateJoystickMode::init();
 
@@ -323,10 +348,14 @@ void innerMain(void *closure,int argc,char **argv) {
   /* Main event loop */
   /*                 */
 
+
+
+
   double oldTime,t,td;
   oldTime=((double)SDL_GetTicks()) /1000.0 ;
   SDL_WarpMouse(screenWidth/2,screenHeight/2);
   while(is_running) {
+
 	t=((double)SDL_GetTicks()) /1000.0 ;
 	td = t - oldTime;
 	oldTime = t;
@@ -343,7 +372,7 @@ void innerMain(void *closure,int argc,char **argv) {
 	Font::tick(td);
 
 	/* Update world */
-	if(GameMode::current) GameMode::current->idle(td);	
+	if(GameMode::current) GameMode::current->idle(td);
 
 	/* Make sure music is still playing */
 	soundIdle();
@@ -357,15 +386,32 @@ void innerMain(void *closure,int argc,char **argv) {
 	//Font::draw();
 	SDL_GL_SwapBuffers();
 
+	/* Expensive computations has to be done *after* tick+draw to keep world in good
+	   synchronisation. */
+	if(GameMode::current) GameMode::current->doExpensiveComputations();
+
+	/*                */
 	/* Process events */
+	/*                */
+
+	/* Joystick - this was a bug since joystick data are also queried using SDL_JoystickUpdate */
+	/*
+	if(Settings::settings->hasJoystick()) SDL_JoystickEventState(SDL_ENABLE); 
+	else SDL_JoystickEventState(SDL_DISABLE); 
+	*/
+
 	SDL_MouseButtonEvent *e=(SDL_MouseButtonEvent*)&event;
-	while(SDL_PollEvent(&event)) 
+	while(SDL_PollEvent(&event)) {
 	  switch(event.type) {
-	  case SDL_QUIT: is_running=0; break;
+	  case SDL_QUIT: 
+	    is_running=0; 
+	    break;
 	  case SDL_MOUSEBUTTONDOWN:
-		if(GameMode::current) GameMode::current->mouseDown(e->button,e->x,e->y);
-		break;
+	    if(GameMode::current) GameMode::current->mouseDown(e->button,e->x,e->y);
+	    break;
 	  case SDL_KEYDOWN:
+	    if(event.key.keysym.sym == 'q') exit(0);
+
 		if(event.key.keysym.sym == SDLK_ESCAPE) {
 		  if(editMode) {
 			((EditMode*)GameMode::current)->askQuit();
@@ -376,6 +422,7 @@ void innerMain(void *closure,int argc,char **argv) {
 		else if(GameMode::current) GameMode::current->key(event.key.keysym.sym);
 		break;
 	  }
+	}
   }
 
   Settings::settings->closeJoystick();
@@ -383,54 +430,54 @@ void innerMain(void *closure,int argc,char **argv) {
   if(GameMode::current)
 	delete(GameMode::current); 
 
+  /* TODO. Delete all gamemodes, including the editor */
   SDL_Quit();
 }
 
 int main(int argc,char **argv) {
   int i;
-
   char guileLoadPath[256+16];/*longest effective share directory plus"GUILE_LOAD_PATH="*/
-
   program_name = argv[0];
 
   /*** Autmatic detection of SHARE_DIR ***/
   effectiveShareDir[0]=0;
   /* From environment variable */
   char *evar=getenv("TRACKBALLS"); 
-  if(evar && strlen(evar) > 0) sprintf(effectiveShareDir,"%s",evar);
+  if(evar && strlen(evar) > 0)
+       snprintf(effectiveShareDir,sizeof(effectiveShareDir)-1,"%s",evar);
   //printf("Looking for %s\n", effectiveShareDir);
   if(!testDir()) {     
        char thisDir[256];
        /* From arg0/share/trackballs  */
-       sprintf(thisDir,"%s",program_name);
+       snprintf(thisDir,sizeof(thisDir),"%s",program_name);
        for(i=strlen(thisDir)-1;i>=0;i--)
          if(thisDir[i] == '/'
 #ifdef WIN32
             || thisDir[i] == '\\'
 #endif
          ) break;
-       thisDir[i] = 0;
+       if ( i >= 0 ) thisDir[i] = 0;
 
        /*If no directory breaks are found just use the current directory*/
-       if( i == 0 ) sprintf(thisDir,".");
+       if( i <= 0 ) snprintf(thisDir,sizeof(thisDir),".");
 
-       sprintf(effectiveShareDir,"%s/share/trackballs",thisDir);
+       snprintf(effectiveShareDir,sizeof(effectiveShareDir),"%s/share/trackballs",thisDir);
 
        if(!testDir()) {
          /* From arg0/../share/trackballs */
-         sprintf(effectiveShareDir,"%s/../share/trackballs",thisDir);
+         snprintf(effectiveShareDir,sizeof(effectiveShareDir),"%s/../share/trackballs",thisDir);
 
          if(!testDir()) {
            /* From arg0/share */
-           sprintf(effectiveShareDir,"%s/share",thisDir);
+           snprintf(effectiveShareDir,sizeof(effectiveShareDir),"%s/share",thisDir);
 
            if(!testDir()) {
              /* From arg0/../share */
-             sprintf(effectiveShareDir,"%s/../share",thisDir);
+             snprintf(effectiveShareDir,sizeof(effectiveShareDir),"%s/../share",thisDir);
 
              if(!testDir()) {
                /* From compilation default */
-               sprintf(effectiveShareDir,"%s",SHARE_DIR_DEFAULT);
+               snprintf(effectiveShareDir,sizeof(effectiveShareDir),"%s",SHARE_DIR_DEFAULT);
 
                if(!testDir()) {
                  printf("Error. Could not find resource directory(%s)\n",effectiveShareDir);
@@ -449,11 +496,31 @@ int main(int argc,char **argv) {
 
   if(NULL == getenv("GUILE_LOAD_PATH"))
   {
-     sprintf(guileLoadPath,"GUILE_LOAD_PATH=%s",effectiveShareDir);
-       
+     snprintf(guileLoadPath,sizeof(guileLoadPath),"GUILE_LOAD_PATH=%s",effectiveShareDir);       
      putenv(guileLoadPath);
   }
 
+#ifdef WIN32
+  if(NULL == getenv("HOME")){
+    char homePath[MAX_PATH];
+    char homeEnv[MAX_PATH + 5];
+    SHGetFolderPath(NULL, CSIDL_APPDATA, NULL, 0, homePath);
+    snprintf(homeEnv,sizeof(homeEnv),"HOME=%s", homePath);
+    for(int i=strlen(homeEnv)-1;i>=0;i--)
+      if( homeEnv[i] == '\\'
+	  ) homeEnv[i] = '/';    
+    //printf("'%s'\n", homeEnv);
+    putenv(homeEnv);
+  }
+#endif
+
+  /* Initialialize i18n with system defaults, before anything else (eg. settings) have been loaded */
+  setlocale(LC_ALL, ""); 
+  char localedir[256];
+  snprintf(localedir,255,"%s/locale",effectiveShareDir);
+  bindtextdomain(PACKAGE,localedir);
+  textdomain(PACKAGE);
+
   scm_boot_guile(argc,argv,innerMain,0);
   return 0;
 }
