@@ -35,16 +35,22 @@
 using namespace std;
 
 Player::Player(Gamer *gamer) :Ball() {
-  inTheAir=false;
+  inTheAir=0;
+  inPipe=0;
   lives = 3;
   realRadius = 0.3; radius = realRadius;
-  resolution = max(12,8+2*Settings::settings->gfx_details);
+
+  ballResolution=BALL_HIRES;
 
   /* Change our color to red */
   primaryColor[0] = 1.0;
   primaryColor[1] = 0.2;
   primaryColor[2] = 0.2;
 
+  specularColor[0] = 1.0;
+  specularColor[1] = 1.0;
+  specularColor[2] = 1.0;
+
   timeLeft = 5;
   bounceFactor=.7;
   crashTolerance = 7;
@@ -56,6 +62,7 @@ Player::Player(Gamer *gamer) :Ball() {
   moveBurst=0.0;
 
   lastJoyX=lastJoyY=0;
+  setReflectivity(0.4,0);
 }
 Player::~Player() {
 }
@@ -65,13 +72,16 @@ void Player::draw() {
 }
 void Player::tick(Real t) {
   double dx,dy;
+  int superAccelerate=0;
+  static time_t lastTick=0;
 
   if(!Game::current) return;
   if(!playing) return;
+
+  dx=0.0; dy=0.0;
   Map *map = Game::current->map;
   health+=t * 0.4; if(health > 1.0) health = 1.0;
 
-  static time_t lastTick;
   time_t thisTick = time(NULL);
   if(thisTick != lastTick && !hasWon) {
 	timeLeft--;
@@ -80,23 +90,21 @@ void Player::tick(Real t) {
   lastTick=thisTick;
   if(timeLeft < 1) { die(DIE_TIMEOUT); return; }
 
-  // when we are in a pipe we might be below ground and thus below the water level even though it's not supposed to
-  // be water here.
+  /* Check for oxygen by seeing if we are below water level. However:
+	 when we are in a pipe we might be below ground and thus below the water level even though 
+	 it's not supposed to be water here.
+  */
   if(map->getWaterHeight(position[0],position[1]) > position[2]+radius*0.75 &&
 	 (!(inPipe && position[2] < map->getHeight(position[0],position[1])))) {
 	oxygen -= (t / (12.0 - 2.0 * Settings::settings->difficulty)) / Game::current->oxygenFactor;
 	if(oxygen <= 0.0) die(DIE_OTHER);
   } else oxygen = min(1.0,oxygen+(t/4.0)/Game::current->oxygenFactor);  
 
-  /* Cap is the maximum acceleration we are allowed to perform (on average) per second */
-  double cap=140.0 - 10.0 * Settings::settings->difficulty;
-  if(modTimeLeft[MOD_SPEED]) {
-	dx = (int) (dx*1.5); dy = (int) (dy*1.5); cap = 200.0;
-  }
-  if(modTimeLeft[MOD_DIZZY]) cap = cap / 2.0;
-
   /* Joysticks */
   if(Settings::settings->hasJoystick()) {
+	if(Settings::settings->joystickButton(0)) key(' ');
+	if(Settings::settings->joystickButton(1)) superAccelerate=1;
+
 	double joyX=Settings::settings->joystickX();
 	double joyY=Settings::settings->joystickY();
 	/* Help keep the joysticks centered if neccessary */
@@ -108,23 +116,36 @@ void Player::tick(Real t) {
   }
 
   /* Give only *relative* mouse movements */
-  int x,y;
-  Uint8 mouseState=SDL_GetMouseState(&x,&y);
-  SDL_GetMouseState(&x,&y);
-  x -= screenWidth/2;
-  y -= screenHeight/2;
-  //if(x != 0 || y != 0)
-  SDL_WarpMouse(screenWidth/2,screenHeight/2);	
-  if(x || y) {
-	dx = x * Settings::settings->mouseSensitivity * 0.1;
-	dy = y * Settings::settings->mouseSensitivity * 0.1;
+  if(!Settings::settings->ignoreMouse) {
+	int mouseX, mouseY;
+
+	Uint8 mouseState=SDL_GetMouseState(&mouseX,&mouseY);
+	mouseX -= screenWidth/2;
+	mouseY -= screenHeight/2;
+	SDL_WarpMouse(screenWidth/2,screenHeight/2);
+
+	/*
+	Uint8 mouseState=SDL_GetRelativeMouseState(&mouseX,&mouseY);
+	*/
+
+ /* DEBUGGING */
+#ifdef FOO
+	printf("Screen Width/2=%d, screenHeight/2=%d\n",screenWidth/2,screenHeight/2);
+	{
+	  int tmp1,tmp2; 
+	  SDL_GetMouseState(&tmp1,&tmp2); 
+	  if(tmp1 != screenWidth/2 || tmp2 != screenHeight/2) fprintf(stderr,"Error - broken SDL_warp %d %d\n",tmp1,tmp2);
+	  if(tmp1 != screenWidth/2 || tmp2 != screenHeight/2) fprintf(stderr,"Again error - broken SDL_warp %d %d\n",tmp1,tmp2);
+	}
+#endif
+
+	if(mouseState & SDL_BUTTON_RMASK) superAccelerate=1;
+	if((mouseX || mouseY)) {
+	  dx = mouseX * Settings::settings->mouseSensitivity * 0.1;
+	  dy = mouseY * Settings::settings->mouseSensitivity * 0.1;
+	}
   }
 
-  //Uint8 mouseState=SDL_GetRelativeMouseState(&x,&y);
-  //SDL_WarpMouse(screenWidth/2,screenHeight/2);	
-
-
-
   /* Handle keyboard steering */
   Uint8 *keystate = SDL_GetKeyState(NULL);
   int shift = SDL_GetModState() & (KMOD_LSHIFT | KMOD_RSHIFT);
@@ -138,6 +159,7 @@ void Player::tick(Real t) {
   if(keystate[SDLK_KP9] || keystate[SDLK_w]) { dx=kscale*t; dy=-kscale*t; }
   if(keystate[SDLK_KP1] || keystate[SDLK_a]) { dx=-kscale*t; dy=kscale*t; }
   if(keystate[SDLK_KP3] || keystate[SDLK_s]) { dx=kscale*t; dy=kscale*t; }
+  if(keystate[SDLK_KP_ENTER] || keystate[SDLK_RETURN]) superAccelerate=1;
 
   /* rotate control as by settings->rotateArrows */
   {
@@ -147,6 +169,23 @@ void Player::tick(Real t) {
 	dx=tmp;
   }
 
+  /* rotate controls if the camera perspective is rotated */
+  double angle=((MainMode*)GameMode::current)->xyAngle*M_PI/2.;
+  if(angle) {
+	double tmp=dx*cos(angle)-dy*sin(angle);
+	dy=dy*cos(angle)+dx*sin(angle); 
+	dx=tmp;
+  }
+
+  /* Cap is the maximum normal acceleration we are allowed to perform (on average) per second */
+  double cap=140.0 - 10.0 * Settings::settings->difficulty;
+  if(modTimeLeft[MOD_SPEED]) {
+	dx = (int) (dx*1.5); dy = (int) (dy*1.5); cap = 200.0;
+  }
+  if(modTimeLeft[MOD_DIZZY]) cap = cap / 2.0;
+
+  /* Limit size of actual movement according to "cap" */
+  /* Uses a trick to avoid capping away movements from integer only devices (eg. mouse) while having a high FPS */
   moveBurst=min(cap*0.2,moveBurst+cap*t);
   double len=sqrt(dx*dx+dy*dy);
   if(len>moveBurst) {
@@ -156,12 +195,6 @@ void Player::tick(Real t) {
   }
   moveBurst -= len;
 
-  /*
-  if(dx > cap * t) dx = cap * t;
-  else if(dx < -cap * t) dx = -cap * t;
-  if(dy > cap * t) dy = cap * t;
-  else if(dy < -cap * t) dy = -cap * t;*/
-
   /*
   if(modTimeLeft[MOD_DIZZY]) {
 	double tmp = dx * sin(Game::current->gameTime*M_PI*0.1) + dy * cos(Game::current->gameTime*M_PI*0.1);
@@ -169,22 +202,39 @@ void Player::tick(Real t) {
 	dx = tmp;
 	}*/
 
-  /* rotate controlls if needed */
-  double angle=((MainMode*)GameMode::current)->xyAngle*M_PI/2.;
-  if(angle) {
-	double tmp=dx*cos(angle)-dy*sin(angle);
-	dy=dy*cos(angle)+dx*sin(angle); 
-	dx=tmp;
-  }
+  /* Do the movement. Also, if nitro is active then create debris after the ball */
 
-  if(!modTimeLeft[MOD_FROZEN] && !hasWon) {
+  if(!modTimeLeft[MOD_FROZEN] && !hasWon && is_on) {
     rotation[0] -= 0.025 * (dy - dx);
     rotation[1] -= 0.025 * (dx + dy);
+    
+    if(superAccelerate && modTimeLeft[MOD_NITRO]) {
+      double dirX=rotation[0];
+      double dirY=rotation[1];
+      double len=sqrt(dirX*dirX+dirY*dirY);
+      static double nitroDebris=0.0;
+      nitroDebris+=t;
+      while(nitroDebris > 0.1) {
+		nitroDebris-=0.1;
+		Coord3d pos,vel;
+		Debris *d=new Debris(this,position,velocity,1.0+frandom()*2.0);
+		d->position[0] += (frandom()-0.5)*radius - velocity[0]*0.1;
+		d->position[1] += (frandom()-0.5)*radius - velocity[1]*0.1;
+		d->position[2] += (frandom()-0.5)*radius;
+		d->modTimeLeft[MOD_GLASS]=-1.0;
+		d->primaryColor[0]=0.1;
+		d->primaryColor[1]=0.6;
+		d->primaryColor[2]=0.1;
+      }
+      if(len>0.0) {
+		dirX /= len;
+		dirY /= len;
+		rotation[0] += dirX * t * 10.0;
+		rotation[1] += dirY * t * 10.0;
+      }
+    }
   }
 
-  /* Jump with left mousebutton */
-  //  if(mouseState & SDL_BUTTON_LMASK) jump();
-
   if(!hasWon) friction = 1.0;
   else friction = 10.0;
   Ball::tick(t);
@@ -241,7 +291,7 @@ void Player::die(int how) {
 		vel[0] = velocity[0] + (sink?0.1:0.5) * 1/2048.0 * ((rand()%2048)-1024);
 		vel[1] = velocity[1] + (sink?0.1:0.5) * 1/2048.0 * ((rand()%2048)-1024);
 		vel[2] = velocity[2] + (sink?0.01:0.5) * 1/2048.0 * ((rand()%2048)-1024);
-		new Debris(this,pos,vel);
+		Debris *d = new Debris(this,pos,vel,2.0+8.0*frandom());
 	  }
   
   if(how == DIE_CRASH)
@@ -255,22 +305,28 @@ void Player::die(int how) {
   else ((MainMode*)GameMode::current)->playerDie();
   alive=0;
 }
-void Player::restart(Coord3d pos) {
-  int i;
 
+void Player::setStartVariables() {
   sink=0.0;
   rotation[0] = rotation[1] = 0.;
-  assign(pos,position);
-  position[2] = Game::current->map->getHeight(position[0],position[1]) + radius;
   zero(velocity);
   playing = true;
   alive=1;
   hasWon=0;
   health=1.0;
   oxygen=1.0;
-  inTheAir=false;
-  modTimeLeft[MOD_DIZZY]=0.0;
+  inTheAir=0;
+  inPipe=0;
   moveBurst=0.0;
+}
+
+void Player::restart(Coord3d pos) {
+  int i;
+
+  setStartVariables();
+  assign(pos,position);
+  position[2] = Game::current->map->getHeight(position[0],position[1]) + radius;
+  modTimeLeft[MOD_DIZZY]=0.0;
 
   /* reset all mods */
   /*for(i=0;i<NUM_MODS;i++)
