@@ -27,6 +27,14 @@
 #include <dirent.h>
 using namespace std;
 
+/* These are the names/codes codes of languages available from the settings
+   menu. The name of languages should appear in the respective native
+   language. Eg. the Swedish language is written as 'Svenska'
+*/
+char *Settings::languageCodes[5]={"","sv_SE","de_DE","it_IT","fr_FR"};
+char *Settings::languageNames[5]={"Default","Svenska","Deutch","Italiano","Francais"};
+int Settings::nLanguages=5;
+
 Settings *Settings::settings;
 void Settings::init() {settings=new Settings();}
 Settings::Settings() {
@@ -45,7 +53,10 @@ Settings::Settings() {
   joy_up=-32767;
   joy_down=32767;
   lastJoyX=0; lastJoyY=0;
-  
+  ignoreMouse=0;
+  doReflections=0;
+  language=0;
+
   /* Load all settings here */
   char str[256];
   snprintf(str,sizeof(str)-1,"%s/.trackballs/settings",getenv("HOME"));
@@ -73,6 +84,13 @@ Settings::Settings() {
 	if(version >= 4) {
 	  fread(&rotateSteering,sizeof(int),1,fp);
 	}
+	if(version >= 5) {
+	  fread(&ignoreMouse,sizeof(int),1,fp);
+	  fread(&doReflections,sizeof(int),1,fp);
+	}
+	if(version >= 6) {
+	  fread(&language,sizeof(int),1,fp);
+	}
 	/* TODO. Santitycheck of values */
 	fclose(fp);
   }
@@ -83,17 +101,17 @@ void Settings::loadLevelSets() {
   nLevelSets=0;
 
   /* ugly fix to make levelset lv.set the first level set */
-  sprintf(str,"%s/levels/lv.set",SHARE_DIR);
+  snprintf(str,sizeof(str),"%s/levels/lv.set",SHARE_DIR);
   loadLevelSet(str);
 
-  sprintf(str,"%s/levels",SHARE_DIR);
+  snprintf(str,sizeof(str),"%s/levels",SHARE_DIR);
   DIR *dir = opendir(str);
   if(!dir) { printf("Can't find the %s/ directory\n",str); exit(0); }
   struct dirent *dirent;
   while((dirent = readdir(dir))) {
 	if(strlen(dirent->d_name) > 4 && strcmp(&dirent->d_name[strlen(dirent->d_name)-4],".set") == 0) {
 	  if(strcmp(dirent->d_name,"lv.set")) {
-		sprintf(str,"%s/levels/%s",SHARE_DIR,dirent->d_name);
+		snprintf(str,sizeof(str),"%s/levels/%s",SHARE_DIR,dirent->d_name);
 		loadLevelSet(str);
 	  }
 	}
@@ -137,16 +155,24 @@ void Settings::loadLevelSet(char *str) {
 
 void Settings::save() {
   char str[256];
-  int version=4;
+  int version=6;
 
   snprintf(str,sizeof(str)-1,"%s/.trackballs",getenv("HOME"));
+  if(pathIsLink(str)) { 
+	fprintf(stderr,_("Error, %s is a symbolic link. Cannot save settings\n"),str);
+	return;
+  }
 
   mkdir(str,S_IXUSR|S_IRUSR|S_IWUSR|S_IXGRP|S_IRGRP|S_IWGRP);
   snprintf(str,sizeof(str)-1,"%s/.trackballs/settings",getenv("HOME"));
+  if(pathIsLink(str)) { 
+	fprintf(stderr,_("Error, %s is a symbolic link. Cannot save settings\n"),str);
+	return;
+  }
   
   /* TODO. Save all settings here */
   FILE *fp = fopen(str,"wb");
-  if(!fp) { printf("Warning. Could not save settins\n"); }
+  if(!fp) { printf("Warning. Could not save settings.\n"); }
   else {
 	/* Version 1 */
 	fwrite(&version,sizeof(int),1,fp);
@@ -168,6 +194,11 @@ void Settings::save() {
 	fwrite(&joy_down,sizeof(int),1,fp);
 	/* Version 4 */
 	fwrite(&rotateSteering,sizeof(int),1,fp);
+	/* Version 5 */
+	fwrite(&ignoreMouse,sizeof(int),1,fp);
+	fwrite(&doReflections,sizeof(int),1,fp);
+	/* Version 6 */
+	fwrite(&language,sizeof(int),1,fp);
 	fclose(fp);
   }
 }
@@ -249,8 +280,24 @@ double Settings::joystickY() {
 int Settings::joystickButton(int n) {
   if(!joystickIndex) return 0;
   SDL_JoystickUpdate();
-  return SDL_JoystickGetButton(joystick,0) || 
+  return SDL_JoystickGetButton(joystick,n);
+
+  /*|| 
 	SDL_JoystickGetButton(joystick,1) || 
 	SDL_JoystickGetButton(joystick,2) || 
-	SDL_JoystickGetButton(joystick,3);  
+	SDL_JoystickGetButton(joystick,3);  */
+}
+void Settings::setLocale() {
+  /* check if the correct language directory exists, otherwise use default */
+  if(language != 0) {
+	char localedir[256];
+	snprintf(localedir,255,"%s/locale/%c%c",effectiveShareDir,languageCodes[language][0],languageCodes[language][1]);
+	if(!dirExists(localedir)) {
+	  fprintf(stderr,"Warning: locale directory %s missing.\n",localedir);
+	  //language=0;
+	}
+  }
+
+  /* Set the locale */
+  setlocale(LC_MESSAGES,languageCodes[language]);  
 }
