@@ -1,5 +1,5 @@
-/* template.h
-   Used to create new header files
+/* settings.h
+   Contains all user specified settings for the game
 
    Copyright (C) 2000  Mathias Broxvall
 
@@ -51,7 +51,9 @@ class Settings {
   int difficulty;
   int rotateSteering;
   int ignoreMouse;
+  int doReflections;
 
+  void setLocale();
   int hasJoystick();
   void closeJoystick();
   int joystickRawX();
@@ -69,6 +71,10 @@ class Settings {
   int joystickIndex,joy_left,joy_right,joy_up,joy_down,joy_center[2];
   int lastJoyX,lastJoyY;
 
+  int language;
+  static char *languageNames[], *languageCodes[];
+  static int nLanguages;
+
  private:
   void loadLevelSet(char *name);
 };
