@@ -34,6 +34,7 @@
 #include "map.h"
 #include "hofMode.h"
 #include "font.h"
+#include "menusystem.h"
 
 #define SETUP_NAME   0
 #define NUM_FIELDS   1
@@ -44,20 +45,15 @@ using namespace std;
 #define Y_DELTA 40
 #define Y_IMG_DIFF 52
 
-#define OPT_NAME       0
-#define OPT_LEVEL_SET  1
-#define OPT_START_LV   2
-#define OPT_DIFFICULTY 3
-#define NUM_OPT        4
-#define OPT_START      5
-#define OPT_COLOR      6
+#define CODE_NAME       1
+#define CODE_LEVEL_SET  2
+#define CODE_START_LV   3
+#define CODE_DIFFICULTY 4
+#define CODE_START      5
+#define CODE_COLOR      6
 
-SetupMode *SetupMode::setupMode;
+SetupMode *SetupMode::setupMode=NULL;
 SDL_Surface *SetupMode::background;
-SDL_Surface *SetupMode::labels[4];
-SDL_Surface *SetupMode::startSurface;
-SDL_Surface *SetupMode::startHigh;
-SDL_Surface *SetupMode::title;
 
 GLfloat colors[5][3] = {{1.0,0.2,0.2}, {0.2,1.0,0.2}, {0.5,0.5,1.0}, {1.0,1.0,0.2},{1.0,1.0,1.0}};
 
@@ -68,29 +64,11 @@ void SetupMode::init() {
 
   if(low_memory) background=NULL;
   else {
-	sprintf(str,"%s/images/setupBackground.jpg",SHARE_DIR);
+	snprintf(str,sizeof(str),"%s/images/setupBackground.jpg",SHARE_DIR);
 	background=IMG_Load(str);
-	if(!background) { printf("Error: Failed to load %s\n",str); exit(0); } 
+	if(!background) { printf(_("Error: Failed to load %s\n"),str); exit(0); } 
   }
 
-  for(i=0;i<4;i++) {
-	sprintf(str,"%s/images/setup%d.png",SHARE_DIR,i+1);
-	labels[i]=IMG_Load(str);
-	if(!labels[i]) { printf("Error: Failed to load %s\n",str); exit(0); } 
-  }
-
-  sprintf(str,"%s/images/start.png",SHARE_DIR);
-  startSurface=IMG_Load(str);
-  if(!startSurface) { printf("Error: Failed to load %s\n",str); exit(0); } 
-
-  sprintf(str,"%s/images/startHigh.png",SHARE_DIR);
-  startHigh=IMG_Load(str);
-  if(!startHigh) { printf("Error: Failed to load %s\n",str); exit(0); } 
-
-  sprintf(str,"%s/images/setupTitle.png",SHARE_DIR);
-  title=IMG_Load(str);
-  if(!title) { printf("Error: Failed to load %s\n",str); exit(0); } 
-
   setupMode=new SetupMode();
 }
 SetupMode::SetupMode() {gamer=new Gamer();levelSet=level=0;}
@@ -99,6 +77,7 @@ void SetupMode::display() {
   glClearColor(0.0,0.0,0.0,0.0);
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
+  /* Draw background */
   Enter2DMode();
   glColor4f(1.0,1.0,1.0,1.0);
   glBindTexture(GL_TEXTURE_2D, texture);
@@ -109,68 +88,72 @@ void SetupMode::display() {
   glTexCoord2f(texMinX, texMaxY); glVertex2i(screenWidth, screenHeight);
   glEnd();
   Leave2DMode();
- 
+
   char str[256];
   int opt;
   Settings *settings = Settings::settings;
   int w,h;
+  int fontSize=48;
+  int titleFontSize=64;
+  int col0 = screenWidth/2 - 400;
+  int col1 = screenWidth/2 + 100;
+  int row0 = screenHeight/2 - 100;
+  int rowSep = fontSize+5;
 
-  drawSurface(title,screenWidth/2-title->w/2,64,title->w,title->h);
+  /* Draw title */
+  addText_Center(0,titleFontSize/2,64,_("Setup new game"),screenWidth/2);
 
-  /* Name */
-  sprintf(str,"%s",gamer->name);
-  displayMenu(labels[0],str,OPT_NAME);
+  /* Name */  
+  clearSelectionAreas();
+  addText_Left(0,fontSize/2,row0+rowSep*0,_("Player Name"),col0);
+  addText_Left(CODE_NAME,fontSize/2,row0+rowSep*0,gamer->name,col1);
 
   /* Level set */
+  addText_Left(0,fontSize/2,row0+rowSep*1,_("Level Set"),col0);
   if(settings->doSpecialLevel)
-	sprintf(str,"N/A");
+	addText_Left(0,fontSize/2,row0+rowSep*1,_("N/A"),col1);
   else
-	sprintf(str,"%s",settings->levelSets[levelSet].name);
-  displayMenu(labels[1],str,OPT_LEVEL_SET);
+	addText_Left(CODE_LEVEL_SET,fontSize/2,row0+rowSep*1,settings->levelSets[levelSet].name,col1);
 
   /* Start level */
+  addText_Left(0,fontSize/2,row0+rowSep*2,_("Level"),col0);
   if(settings->doSpecialLevel)
-	sprintf(str,"%s",settings->specialLevel);
-  else {
-	sprintf(str,"%s",gamer->levels[levelSet][level].name);
-  }
-  displayMenu(labels[2],str,OPT_START_LV);
+	addText_Left(0,fontSize/2,row0+rowSep*2,settings->specialLevel,col1);
+  else 
+	addText_Left(CODE_START_LV,fontSize/2,row0+rowSep*2,gamer->levels[levelSet][level].name,col1);
 
   /* Difficulty */
-  sprintf(str,"%s",(settings->difficulty==0?"Easy":(settings->difficulty==1?"Normal":"Hard")));
-  displayMenu(labels[3],str,OPT_DIFFICULTY);
+  addText_Left(0,fontSize/2,row0+rowSep*3,_("Difficulty"),col0);
+  addText_Left(CODE_DIFFICULTY,fontSize/2,row0+rowSep*3,(settings->difficulty==0?_("Easy"):(settings->difficulty==1?_("Normal"):_("Hard"))),col1);
 
   /* Start */
-  if(selected == OPT_START)
-	drawSurface(startHigh,screenWidth/2-startHigh->w/2,screenHeight-40-Y_IMG_DIFF,startHigh->w,startHigh->h);
-  else
-	drawSurface(startSurface,screenWidth/2-startSurface->w/2,screenHeight-40-Y_IMG_DIFF,startSurface->w,startSurface->h);
-  glColor4f(1.0,1.0,1.0,1.0);
+  addText_Center(CODE_START,fontSize/2,screenHeight-64,_("Start Game"),screenWidth/2);
 
   /* Info */
   int INFO_BASE=screenHeight - 70;
   int INFO_DELTA=20;
   int INFO_RIGHT=20;
   double INFO_WIDTH=10.0, INFO_HEIGHT=10.0, INFO_R=1.0, INFO_G=1.0, INFO_B=1.0, INFO_A=1.0;
-  sprintf(str,"Total games played: %d", gamer->timesPlayed);
-  //draw2DString(infoFont,str,10,INFO_BASE+INFO_DELTA*0,0,0,0);
+  snprintf(str,sizeof(str),_("Total games played: %d"), gamer->timesPlayed);
   Font::drawSimpleText(0, str, INFO_RIGHT, INFO_BASE+INFO_DELTA*0, INFO_WIDTH, INFO_HEIGHT, INFO_R, INFO_G, INFO_B, INFO_A); 
   if(gamer->timesPlayed)
-	sprintf(str,"Average score: %d", gamer->totalScore/gamer->timesPlayed);
+	snprintf(str,sizeof(str),_("Average score: %d"), gamer->totalScore/gamer->timesPlayed);
   else
-	sprintf(str,"Average score: N/A");
-  //draw2DString(infoFont,str,10,INFO_BASE+INFO_DELTA*1,0,0,0);
+	snprintf(str,sizeof(str),_("Average score: N/A"));
   Font::drawSimpleText(0, str, INFO_RIGHT, INFO_BASE+INFO_DELTA*1, INFO_WIDTH, INFO_HEIGHT, INFO_R, INFO_G, INFO_B, INFO_A); 
-  sprintf(str,"Levels completed: %d", gamer->nLevelsCompleted);
-  //draw2DString(infoFont,str,10,INFO_BASE+INFO_DELTA*2,0,0,0);
+  snprintf(str,sizeof(str),_("Levels completed: %d"), gamer->nLevelsCompleted);
   Font::drawSimpleText(0, str, INFO_RIGHT, INFO_BASE+INFO_DELTA*2, INFO_WIDTH, INFO_HEIGHT, INFO_R, INFO_G, INFO_B, INFO_A); 
 
+
+  /*                      */
+  /* Draw the player ball */
+  /*                      */
+
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glTranslatef(0.75, -0.75, 0.0); // 0.7
   gluPerspective(40,640.0/480.0,1.0,1e20);
 
-
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
   gluLookAt(-6.5, -6.5, 10.0,
@@ -187,6 +170,8 @@ void SetupMode::display() {
   glEnable(GL_LIGHT0);
   GLfloat ambient[] = { 0.2, 0.2, 0.2, 1.0 };
   glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient);
+  glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL, GL_SEPARATE_SPECULAR_COLOR);
+
   glShadeModel(GL_SMOOTH);
   glEnable(GL_TEXTURE_2D);
   glBindTexture(GL_TEXTURE_2D, textures[gamer->textureNum]);
@@ -205,13 +190,14 @@ void SetupMode::display() {
 	diffuse[i] = colors[gamer->color][i];
   diffuse[3] = 1.0;
   glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,diffuse);
-  GLfloat specular[4] = {0.5,0.5,0.5,1.0};
+
+  GLfloat specular[4] = {1.0,1.0,1.0,1.0};
   glMaterialfv(GL_FRONT,GL_SPECULAR,specular);
-  glMaterialf(GL_FRONT,GL_SHININESS,20.0);	
+  glMaterialf(GL_FRONT,GL_SHININESS,10.0);	
   glRotated(t*-50.0, 0.0, 1.0, 0.0);
 
   glPushAttrib(GL_ENABLE_BIT|GL_CURRENT_BIT);
-  gluSphere(qball, 0.8, max(10,4+2*settings->gfx_details), max(10,4+2*settings->gfx_details));
+  gluSphere(qball, 0.8, max(10,4+3*settings->gfx_details), max(10,4+3*settings->gfx_details));
   glPopAttrib();
 
   drawMousePointer();
@@ -221,8 +207,9 @@ void SetupMode::display() {
 }
 void SetupMode::key(int key) {
   int len=strlen(gamer->name);
+  int selected = getSelectedArea();
 
-  if(key == SDLK_SPACE && selected != OPT_NAME) { int x,y; SDL_GetMouseState(&x,&y); mouseDown(1,x,y); }
+  if(key == SDLK_SPACE && selected != CODE_NAME) { int x,y; SDL_GetMouseState(&x,&y); mouseDown(1,x,y); }
   if(key == SDLK_RETURN) { int x,y; SDL_GetMouseState(&x,&y); mouseDown(3,x,y); }
 
   if(key == SDLK_BACKSPACE) {
@@ -253,27 +240,17 @@ void SetupMode::idle(Real td) {
   if(keystate[SDLK_UP]) { y-=(int)(150/fps); SDL_WarpMouse(x,y); }
   if(keystate[SDLK_DOWN]) { y+=(int)(150/fps); SDL_WarpMouse(x,y); }
 
-  Settings *settings = Settings::settings;
-  Uint8 mouseState=SDL_GetMouseState(&x,&y); x+=32; y+=32;
-  if(y >= Y_BASE && y <= Y_BASE+Y_DELTA*NUM_OPT)
-	selected = (y-Y_BASE) / Y_DELTA;
-  else if(y >= (screenHeight * 2)/3 && x > (screenWidth * 3)/4) selected = OPT_COLOR;
-  else if(y > screenHeight - 40 && y < screenHeight - 40 + Y_DELTA) selected = OPT_START;
-  else selected = -1;
-
   t += td;
 }
 void SetupMode::activated() {  
   char str[256];
  
-  selected=OPT_NAME;
-  SDL_WarpMouse(50,Y_BASE+Y_DELTA*selected-Y_DELTA/2);
   gamer->reloadNames();
 
   if(!background) {
-	sprintf(str,"%s/images/setupBackground.jpg",SHARE_DIR);
+	snprintf(str,sizeof(str),"%s/images/setupBackground.jpg",SHARE_DIR);
 	background=IMG_Load(str);
-	if(!background) { printf("Error: Failed to load %s\n",str); exit(0); } 
+	if(!background) { printf(_("Error: Failed to load %s\n"),str); exit(0); } 
   }
 
   /* Preloads the background texture. */
@@ -306,45 +283,31 @@ void SetupMode::start() {
   }
   GameMode::activate(MainMode::mainMode);
 }
-void SetupMode::displayMenu(SDL_Surface *image,char *str,int opt) {
-  drawSurface(image,screenWidth/2-310,Y_BASE+opt*Y_DELTA-Y_IMG_DIFF,image->w,image->h);
 
-  if(str && strlen(str)>0) {
-	if(selected == opt)
-	  //draw2DString(menuFont,str,screenWidth/2+50,Y_BASE+opt*Y_DELTA+4,220,220,220);
-	  Font::drawSimpleText(0,str,screenWidth/2+50,Y_BASE+opt*Y_DELTA-22,20.0,20.0,220/256.,220/256.,220/256.,1.);
-	else
-	  //draw2DString(menuFont,str,screenWidth/2+50,Y_BASE+opt*Y_DELTA+4,220,220,64);
-	  Font::drawSimpleText(0,str,screenWidth/2+50,Y_BASE+opt*Y_DELTA-22,20.0,20.0,220/256.,220/256.,64/256.,1.);
-
-  }
-}
 void SetupMode::mouseDown(int button,int x,int y) {
-  x+=32; y+=32;
-  if(y >= Y_BASE && y <= Y_BASE+Y_DELTA*NUM_OPT)
-	selected = (y-Y_BASE) / Y_DELTA;
-  else if(y >= (screenHeight * 2)/3 && x > (screenWidth * 3)/4) selected = OPT_COLOR;
-  else if(y > screenHeight - 40 && y < screenHeight - 40 + Y_DELTA) selected = OPT_START;
-  else selected = -1;
+  int selected = getSelectedArea();
 
+  x+=32; y+=32;
+  if(y >= (screenHeight * 2)/3 && x > (screenWidth * 3)/4) selected = CODE_COLOR; // Ugly fix for selecting the 3D object
+  
   Settings *settings=Settings::settings;
-  if(selected != -1) {
-	int left=button==1;
+  if(selected) {
+	int left=(button==1);
 	switch(selected) {
-	case OPT_START: start(); break;
-	case OPT_COLOR: 
+	case CODE_START: start(); break;
+	case CODE_COLOR: 
 	  if(left)
 		gamer->color = (gamer->color+1) % 5; 
 	  else
 		{ gamer->color = 4; gamer->textureNum = (gamer->textureNum+1)%numTextures; }
 	  break;
-	case OPT_DIFFICULTY: 
+	case CODE_DIFFICULTY: 
 	  settings->difficulty = mymod((settings->difficulty+(left?1:-1)),3); 
 	  break;
-	case OPT_START_LV: 
+	case CODE_START_LV: 
 	  level = mymod((level+(left?1:-1)),gamer->nKnownLevels[levelSet]);
 	  break;
-	case OPT_NAME: 
+	case CODE_NAME: 
 	  if(gamer->nNames) {
 		name=(name+(left?1:-1))%gamer->nNames; 
 		strncpy(gamer->name,gamer->names[name],20); 
@@ -352,7 +315,7 @@ void SetupMode::mouseDown(int button,int x,int y) {
 		level=0;
 	  }
 	  break;
-	case OPT_LEVEL_SET: 
+	case CODE_LEVEL_SET: 
 	  levelSet = mymod((levelSet+(left?1:-1)),settings->nLevelSets); 
 	  level=0; break;
 	}
