@@ -36,14 +36,13 @@ class SetupMode : public GameMode {
  private:
   void start();
   class Gamer *gamer;
-  int selected,levelSet,level,name;
-  static SDL_Surface *background,*labels[4],*startSurface,*title,*startHigh;
+  int levelSet,level,name;
+  static SDL_Surface *background;
   double t;
 
   GLuint texture;
   GLfloat texMinX, texMinY;
   GLfloat texMaxX, texMaxY;
-  void displayMenu(SDL_Surface *image,char *str,int position);
 };
 
 #endif
